///////////////////////////////////////////////////////////////////////////////
// PjPlatonic class
//
// Last update: 13/01/2008
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Native JAVA imports
///////////////////////////////////////////////////////////////////////////////
import java.awt.Color;

///////////////////////////////////////////////////////////////////////////////
// JavaView imports
///////////////////////////////////////////////////////////////////////////////
import jv.anim.PsAnimation;
import jv.function.PuFunction;
import jv.geom.PgElementSet;
import jv.geom.PgPointSet;
import jv.geom.PgPolygon;
import jvx.geom.PwCleanMesh;
import jvx.geom.PwClip;
import jvx.geom.PwModel;
import jvx.geom.PwUnfold;
import jv.number.PuDouble;
import jv.number.PuInteger;
import jv.project.PgGeometryIf;
import jv.project.PjProject;
import jv.project.PvDisplayIf;
import jv.project.PvPickListenerIf;
import jv.project.PvPickEvent;
import jvx.surface.PgDomainDescr;
import jvx.surface.PgParmSurface;
import jv.vecmath.PdVector;
import jv.vecmath.PiVector;


///////////////////////////////////////////////////////////////////////////////
// Platonic applet
///////////////////////////////////////////////////////////////////////////////
public class PjPlatonic extends PjProject implements PvPickListenerIf
{
   protected int idiom;
   protected int numberOfVertices;
   protected int numberOfEdges;
   protected int numberOfElements;

   protected String[] tetrahedron_trans = {"O Tetraedro Regular",
                                           "The Regular Tetrahedron"};
   protected String[] cube_trans = {"O Cubo", "The Cube"};
   protected String[] octahedron_trans = {"O Octaedro Regular",
                                          "The Regular Octahedron"};
   protected String[] dodecahedron_trans = {"O Dodecaedro Regular",
                                            "The Regular Dodecahedron"};
   protected String[] icosahedron_trans = {"O Icosaedro Regular",
                                           "The Regular Icosahedron"};

   protected String[] alphabet = {"A", "B", "C", "D", "E", "F", "G", "H", "I",
                                  "J", "K", "L", "M", "N", "O", "P", "Q", "R",
                                  "S", "T", "U", "V", "W", "X", "Y", "Z"};
                                          // Used to restore vertices names
                                          // after a PwWorkshop operation

   protected String solidName;
   protected PgElementSet platonic;

   protected double R = 5.0;              // Circumsphere radius
   protected PgParmSurface circumsphere;
   protected PgParmSurface insphere;
   protected PgPointSet dualvertices;
   protected PgElementSet dual;

   protected PgElementSet tetrahedron_ic; // Tetrahedron inscribed in the cube
   protected PgElementSet icosahedron_io; // Icosahedron inscribed in the
                                          // octahedron.
   protected PgElementSet goldenrectangle1; 
   protected PgElementSet goldenrectangle2; 
   protected PgElementSet goldenrectangle3;
   protected PgElementSet cube_id;
   protected PgElementSet tetrahedron_id;

   protected PgPolygon picked_point;      // Picked point
   protected PdVector rotationAxis;       // Rotation axis
   protected PuDouble translation_factor;
   protected PgElementSet cutplane;

   protected PwClip m_pwClip;
   protected String m_defClipFunc;

   protected PwUnfold m_pwUnfold;
   protected PsAnimation m_pwAnimation;
   protected PuInteger flatten_level;
   protected PvDisplayIf disp; // The PwUnfold needs a reference to the
                               // display in order to work properly.

   protected PwModel m_pwModel;
   protected PuDouble m_parameter;
   protected int m_modelingType;
   protected double m_parameterTrunc;
   protected double m_parameterStellate;
   protected double m_parameterPunch;
   protected double m_truncParamLowerBound;
   protected double m_truncParamUpperBound;
   protected double m_stellateParamLowerBound;
   protected double m_stellateParamUpperBound;
   protected double m_punchParamLowerBound;
   protected double m_punchParamUpperBound;
   protected boolean trn_fill = false;


   //////////////////////////////////////////////////////////////////////////
   // Default constructor
   //////////////////////////////////////////////////////////////////////////
   public PjPlatonic(int input_idiom, String input_solidName)
   {
      super("Platonic Applet");

      idiom = input_idiom;
      solidName = input_solidName;

      if (getClass() == PjPlatonic.class)
      {
         init();
      };
   }; // End of PjPlatonic()

   //////////////////////////////////////////////////////////////////////////
   // Initialize variables
   //////////////////////////////////////////////////////////////////////////
   public void init()
   {
       platonic = new PgElementSet(3); 

       double v = 0.0; // Insphere radius, configured below for each platonic
                       // solid.


       //////////////////////////////////////////////////////////////////////
       // BEGIN OF PART 1 (THE NUMBER OF VERTICES)
       //////////////////////////////////////////////////////////////////////
       // The cube
       if (solidName.equals(cube_trans[idiom]))
       {
          numberOfVertices = 8;
          numberOfEdges = 12;
          numberOfElements = 6;
       };

       // The regular thetrahedron
       if (solidName.equals(tetrahedron_trans[idiom]))
       {
          numberOfVertices = 4;
          numberOfEdges = 6;
          numberOfElements = 4;
       };

       // The regular octahedron
       if (solidName.equals(octahedron_trans[idiom]))
       {
          numberOfVertices = 6;
          numberOfEdges = 12;
          numberOfElements = 8;
       };

       // The regular dodecahedron
       if (solidName.equals(dodecahedron_trans[idiom]))
       {
          numberOfVertices = 20;
          numberOfEdges = 30;
          numberOfElements = 12;
       };

       // The regular icosahedron
       if (solidName.equals(icosahedron_trans[idiom]))
       {
          numberOfVertices = 12;
          numberOfEdges = 30;
          numberOfElements = 20;
       };
       //////////////////////////////////////////////////////////////////////
       // END OF PART 1 
       //////////////////////////////////////////////////////////////////////
       
       
       platonic.setNumElements(numberOfElements); 
       platonic.setNumVertices(numberOfVertices);
       PdVector vertices[] = new PdVector[numberOfVertices];
       PdVector tangentpoints[] = new PdVector[numberOfElements];


       //////////////////////////////////////////////////////////////////////
       // BEGIN OF PART 2 (THE SOLID)
       //////////////////////////////////////////////////////////////////////
       // The cube
       if (solidName.equals(cube_trans[idiom]))
       {
          // Setup vertices coordinates
          v = R/Math.sqrt(3);
          vertices[0] = new PdVector(+v, +v, +v);
          vertices[1] = new PdVector(-v, +v, +v);
          vertices[2] = new PdVector(-v, -v, +v);
          vertices[3] = new PdVector(+v, -v, +v);
          vertices[4] = new PdVector(+v, +v, -v);
          vertices[5] = new PdVector(-v, +v, -v);
          vertices[6] = new PdVector(-v, -v, -v);
          vertices[7] = new PdVector(+v, -v, -v);

   
          // Defining the vertices coordinates
          platonic.setVertex(0, vertices[0]);
          platonic.setVertex(1, vertices[1]);
          platonic.setVertex(2, vertices[2]);
          platonic.setVertex(3, vertices[3]);
          platonic.setVertex(4, vertices[4]);
          platonic.setVertex(5, vertices[5]);
          platonic.setVertex(6, vertices[6]);
          platonic.setVertex(7, vertices[7]);
   
          // Defining the vertices names
          restoreInitialConfiguration();
   
          // Setup faces specifying vertices id order
          platonic.setElement(0, 0, 1, 2, 3);
          platonic.setElement(1, 1, 5, 6, 2);
          platonic.setElement(2, 7, 6, 5, 4);
          platonic.setElement(3, 4, 0, 3, 7);
          platonic.setElement(4, 4, 5, 1, 0);
          platonic.setElement(5, 3, 2, 6, 7);
       };

       // The regular thetrahedron
       if (solidName.equals(tetrahedron_trans[idiom]))
       {
          // Setup vertices coordinates
          v = R/3.0;
          vertices[0] = new PdVector(-R*Math.sqrt(2)/3,
                                     -R*Math.sqrt(2.0/3.0),
                                     -R/3.0);
          vertices[1] = new PdVector(R*2.0*Math.sqrt(2)/3, 0, -R/3.0);
          vertices[2] = new PdVector(-R*Math.sqrt(2)/3,
                                      R*Math.sqrt(6)/3, -R/3.0);
          vertices[3] = new PdVector(0, 0, R); 
          
          // Defining the vertices coordinates
          platonic.setVertex(0, vertices[0]);
          platonic.setVertex(1, vertices[1]);
          platonic.setVertex(2, vertices[2]);
          platonic.setVertex(3, vertices[3]);
   
          // Defining the vertices names
          restoreInitialConfiguration();
   
          // Setup faces specifying vertices id order
          platonic.setElement(0, 2, 1, 0);
          platonic.setElement(1, 0, 1, 3);
          platonic.setElement(2, 1, 2, 3);
          platonic.setElement(3, 2, 0, 3);
       };

       // The regular occtahedron
       if (solidName.equals(octahedron_trans[idiom]))
       {
          // Setup vertices coordinates
          v = Math.sqrt(3)*R/3;
 
          vertices[0] = new PdVector(+R,  0,  0);
          vertices[1] = new PdVector( 0, +R,  0);
          vertices[2] = new PdVector(-R,  0,  0);
          vertices[3] = new PdVector( 0, -R,  0);
          vertices[4] = new PdVector( 0,  0,  R);
          vertices[5] = new PdVector( 0,  0, -R);
   
          // Defining the vertices coordinates
          platonic.setVertex(0, vertices[4]);
          platonic.setVertex(1, vertices[0]);
          platonic.setVertex(2, vertices[1]);
          platonic.setVertex(3, vertices[2]);
          platonic.setVertex(4, vertices[3]);
          platonic.setVertex(5, vertices[5]);
   
          // Defining the vertices names
          restoreInitialConfiguration();
   
          // Setup faces specifying vertices id order
          platonic.setElement(0, 0, 1, 2);
          platonic.setElement(1, 0, 2, 3);
          platonic.setElement(2, 0, 3, 4);
          platonic.setElement(3, 0, 4, 1);
          platonic.setElement(4, 5, 2, 1);
          platonic.setElement(5, 5, 3, 2);
          platonic.setElement(6, 5, 4, 3);
          platonic.setElement(7, 5, 1, 4);
       };

       // The regular dodecahedron
       if (solidName.equals(dodecahedron_trans[idiom]))
       {
          // Setup vertices coordinates
          double w = Math.sqrt(3.0)*R/3.0;
          double p = (Math.sqrt(5.0)+1.0)/2.0;
          v = (w/5)*(Math.sqrt(5+(10*p)+10*Math.pow(p,2)));
          
          vertices[0]  = new PdVector(      w,      w,      w); // A
          vertices[1]  = new PdVector(w*(p-1),      0,    w*p); // B
          vertices[2]  = new PdVector(      w,     -w,      w); // C
          vertices[3]  = new PdVector(    w*p,w*(1-p),      0); // D
          vertices[4]  = new PdVector(    w*p,w*(p-1),      0); // E
          vertices[5]  = new PdVector(     -w,      w,      w); // F
          vertices[6]  = new PdVector(w*(1-p),      0,    w*p); // G
          vertices[7]  = new PdVector(     -w,     -w,      w); // H
          vertices[8]  = new PdVector( w*(-p),w*(1-p),      0); // I
          vertices[9]  = new PdVector( w*(-p),w*(p-1),      0); // J
          vertices[10] = new PdVector(      0, w*(-p),w*(1-p)); // U
          vertices[11] = new PdVector(      0, w*(-p),w*(p-1)); // V
          vertices[12] = new PdVector(     -w,      w,     -w); // Q
          vertices[13] = new PdVector(     -w,     -w,     -w); // T
          vertices[14] = new PdVector(w*(p-1),      0, w*(-p)); // S
          vertices[15] = new PdVector(      w,      w,     -w); // L
          vertices[16] = new PdVector(w*(1-p),      0, w*(-p)); // M
          vertices[17] = new PdVector(      w,     -w,     -w); // N
          vertices[18] = new PdVector(      0,    w*p,w*(p-1)); // O
          vertices[19] = new PdVector(      0,    w*p,w*(1-p)); // P

          // Defining the vertices coordinates
          platonic.setVertex(0, vertices[0]);
          platonic.setVertex(1, vertices[1]);
          platonic.setVertex(2, vertices[2]);
          platonic.setVertex(3, vertices[3]);
          platonic.setVertex(4, vertices[4]);
          platonic.setVertex(5, vertices[5]);
          platonic.setVertex(6, vertices[6]);
          platonic.setVertex(7, vertices[7]);
          platonic.setVertex(8, vertices[8]);
          platonic.setVertex(9, vertices[9]);
          platonic.setVertex(10, vertices[10]);
          platonic.setVertex(11, vertices[11]);
          platonic.setVertex(12, vertices[12]);
          platonic.setVertex(13, vertices[13]);
          platonic.setVertex(14, vertices[14]);
          platonic.setVertex(15, vertices[15]);
          platonic.setVertex(16, vertices[16]);
          platonic.setVertex(17, vertices[17]);
          platonic.setVertex(18, vertices[18]);
          platonic.setVertex(19, vertices[19]);
   
          // Defining the vertices names
          restoreInitialConfiguration();
   
          // Setup faces specifying vertices id order
          PiVector idx00 = new PiVector(5);
          idx00.setEntry(0, 1);
          idx00.setEntry(1, 0);
          idx00.setEntry(2, 18);
          idx00.setEntry(3, 5);
          idx00.setEntry(4, 6);
          platonic.setElement(0, idx00);

          PiVector idx01 = new PiVector(5);
          idx01.setEntry(0, 0);
          idx01.setEntry(1, 4);
          idx01.setEntry(2, 15);
          idx01.setEntry(3, 19);
          idx01.setEntry(4, 18);
          platonic.setElement(1, idx01);

          PiVector idx02 = new PiVector(5);
          idx02.setEntry(0, 3);
          idx02.setEntry(1, 4);
          idx02.setEntry(2, 0);
          idx02.setEntry(3, 1);
          idx02.setEntry(4, 2);
          platonic.setElement(2, idx02);

          PiVector idx03 = new PiVector(5);
          idx03.setEntry(0, 5);
          idx03.setEntry(1, 18);
          idx03.setEntry(2, 19);
          idx03.setEntry(3, 12);
          idx03.setEntry(4, 9);
          platonic.setElement(3, idx03);

          PiVector idx04 = new PiVector(5);
          idx04.setEntry(0, 17);
          idx04.setEntry(1, 14);
          idx04.setEntry(2, 15);
          idx04.setEntry(3, 4);
          idx04.setEntry(4, 3);
          platonic.setElement(4, idx04);

          PiVector idx05 = new PiVector(5);
          idx05.setEntry(0, 6);
          idx05.setEntry(1, 7);
          idx05.setEntry(2, 11);
          idx05.setEntry(3, 2);
          idx05.setEntry(4, 1);
          platonic.setElement(5, idx05);

          PiVector idx06 = new PiVector(5);
          idx06.setEntry(0, 9);
          idx06.setEntry(1, 8);
          idx06.setEntry(2, 7);
          idx06.setEntry(3, 6);
          idx06.setEntry(4, 5);
          platonic.setElement(6, idx06);

          PiVector idx07 = new PiVector(5);
          idx07.setEntry(0, 11);
          idx07.setEntry(1, 10);
          idx07.setEntry(2, 17);
          idx07.setEntry(3, 3);
          idx07.setEntry(4, 2);
          platonic.setElement(7, idx07);

          PiVector idx08 = new PiVector(5);
          idx08.setEntry(0, 8);
          idx08.setEntry(1, 13);
          idx08.setEntry(2, 10);
          idx08.setEntry(3, 11);
          idx08.setEntry(4, 7);
          platonic.setElement(8, idx08);

          PiVector idx09 = new PiVector(5);
          idx09.setEntry(0, 14);
          idx09.setEntry(1, 16);
          idx09.setEntry(2, 12);
          idx09.setEntry(3, 19);
          idx09.setEntry(4, 15);
          platonic.setElement(9, idx09);

          PiVector idx10 = new PiVector(5);
          idx10.setEntry(0, 9);
          idx10.setEntry(1, 12);
          idx10.setEntry(2, 16);
          idx10.setEntry(3, 13);
          idx10.setEntry(4, 8);
          platonic.setElement(10, idx10);

          PiVector idx11 = new PiVector(5);
          idx11.setEntry(0, 10);
          idx11.setEntry(1, 13);
          idx11.setEntry(2, 16);
          idx11.setEntry(3, 14);
          idx11.setEntry(4, 17);
          platonic.setElement(11, idx11);
       };

       // The regular icosahedron
       if (solidName.equals(icosahedron_trans[idiom]))
       {
          // Setup vertices coordinates
  	      double k = R/Math.sqrt(5.0 + 2.0*Math.sqrt(5));
	      double p = (Math.sqrt(5.0)+1.0)/2.0; 
          v = Math.sqrt(3.0)*(3.0 + Math.sqrt(5.0))*R/(3.0*Math.sqrt(10.0 + 2.0*Math.sqrt(5)));
 
          vertices[0]  = new PdVector( k*(p+1),       0,     k*p); // A
          vertices[1]  = new PdVector(k*(-p-1),       0,     k*p); // B
          vertices[2]  = new PdVector(k*(-p-1),       0,  k*(-p)); // C
          vertices[3]  = new PdVector( k*(p+1),       0,  k*(-p)); // D 
          vertices[4]  = new PdVector(       0,     k*p, k*(p+1)); // E 
          vertices[5]  = new PdVector(       0,     k*p,k*(-p-1)); // F
          vertices[6]  = new PdVector(       0,  k*(-p),k*(-p-1)); // G
          vertices[7]  = new PdVector(       0,  k*(-p), k*(p+1)); // H
          vertices[8]  = new PdVector(     k*p, k*(p+1),       0); // I 
          vertices[9]  = new PdVector(     k*p,k*(-p-1),       0); // J 
          vertices[10] = new PdVector(  k*(-p),k*(-p-1),       0); // L 
          vertices[11] = new PdVector(  k*(-p), k*(p+1),       0); // M
         
          // Defining the vertices coordinates
          platonic.setVertex(0, vertices[0]);
          platonic.setVertex(1, vertices[1]);
          platonic.setVertex(2, vertices[2]);
          platonic.setVertex(3, vertices[3]);
          platonic.setVertex(4, vertices[4]);
          platonic.setVertex(5, vertices[5]);
          platonic.setVertex(6, vertices[6]);
          platonic.setVertex(7, vertices[7]);
          platonic.setVertex(8, vertices[8]);
          platonic.setVertex(9, vertices[9]);
          platonic.setVertex(10, vertices[10]);
          platonic.setVertex(11, vertices[11]);
            
          // Defining the vertices names
          restoreInitialConfiguration();
   
          // Setup faces specifying vertices id order
          PiVector idx00 = new PiVector(3);
          idx00.setEntry(0, 4);
          idx00.setEntry(1, 0);
          idx00.setEntry(2, 8);
          platonic.setElement(0, idx00);

          PiVector idx01 = new PiVector(3);
          idx01.setEntry(0, 8);
          idx01.setEntry(1, 0);
          idx01.setEntry(2, 3);
          platonic.setElement(1, idx01);

          PiVector idx02 = new PiVector(3);
          idx02.setEntry(0, 0);
          idx02.setEntry(1, 9);
          idx02.setEntry(2, 3);
          platonic.setElement(2, idx02);

          PiVector idx03 = new PiVector(3);
          idx03.setEntry(0, 4);
          idx03.setEntry(1, 7);
          idx03.setEntry(2, 0);
          platonic.setElement(3, idx03);

          PiVector idx04 = new PiVector(3);
          idx04.setEntry(0, 7);
          idx04.setEntry(1, 9);
          idx04.setEntry(2, 0);
          platonic.setElement(4, idx04);

          PiVector idx05 = new PiVector(3);
          idx05.setEntry(0, 7);
          idx05.setEntry(1, 10);
          idx05.setEntry(2, 9);
          platonic.setElement(5, idx05);

          PiVector idx06 = new PiVector(3);
          idx06.setEntry(0, 10);
          idx06.setEntry(1, 6);
          idx06.setEntry(2, 9);
          platonic.setElement(6, idx06);

          PiVector idx07 = new PiVector(3);
          idx07.setEntry(0, 6);
          idx07.setEntry(1, 3);
          idx07.setEntry(2, 9);
          platonic.setElement(7, idx07);

          PiVector idx08 = new PiVector(3);
          idx08.setEntry(0, 6);
          idx08.setEntry(1, 5);
          idx08.setEntry(2, 3);
          platonic.setElement(8, idx08);

          PiVector idx09 = new PiVector(3);
          idx09.setEntry(0, 3);
          idx09.setEntry(1, 5);
          idx09.setEntry(2, 8);
          platonic.setElement(9, idx09);

          PiVector idx10 = new PiVector(3);
          idx10.setEntry(0, 5);
          idx10.setEntry(1, 11);
          idx10.setEntry(2, 8);
          platonic.setElement(10, idx10);

          PiVector idx11 = new PiVector(3);
          idx11.setEntry(0, 8);
          idx11.setEntry(1, 11);
          idx11.setEntry(2, 4);
          platonic.setElement(11, idx11);

          PiVector idx12 = new PiVector(3);
          idx12.setEntry(0, 11);
          idx12.setEntry(1, 1);
          idx12.setEntry(2, 4);
          platonic.setElement(12, idx12);

          PiVector idx13 = new PiVector(3);
          idx13.setEntry(0, 4);
          idx13.setEntry(1, 1);
          idx13.setEntry(2, 7);
          platonic.setElement(13, idx13);
           
          PiVector idx14 = new PiVector(3);
          idx14.setEntry(0, 1);
          idx14.setEntry(1, 10);
          idx14.setEntry(2, 7);
          platonic.setElement(14, idx14);
           
          PiVector idx15 = new PiVector(3);
          idx15.setEntry(0, 10);
          idx15.setEntry(1, 2);
          idx15.setEntry(2, 6);
          platonic.setElement(15, idx15);
           
          PiVector idx16 = new PiVector(3);
          idx16.setEntry(0, 6);
          idx16.setEntry(1, 2);
          idx16.setEntry(2, 5);
          platonic.setElement(16, idx16);
           
          PiVector idx17 = new PiVector(3);
          idx17.setEntry(0, 5);
          idx17.setEntry(1, 2);
          idx17.setEntry(2, 11);
          platonic.setElement(17, idx17);
           
          PiVector idx18 = new PiVector(3);
          idx18.setEntry(0, 10);
          idx18.setEntry(1, 1);
          idx18.setEntry(2, 2);
          platonic.setElement(18, idx18);
           
          PiVector idx19 = new PiVector(3);
          idx19.setEntry(0, 2);
          idx19.setEntry(1, 1);
          idx19.setEntry(2, 11);
          platonic.setElement(19, idx19);
       };
       //////////////////////////////////////////////////////////////////////
       // END OF PART 2 
       //////////////////////////////////////////////////////////////////////
       

       // Configuring the platonic solid
       platonic.setTransparency(0.5);
       platonic.showVertices(true);
       platonic.showTransparency(true);
       platonic.showElementColors(true);
       platonic.showElements(true);
       platonic.showEdges(true);
       platonic.showVertexOutline(true);
       platonic.showVertexOutline(true);
       platonic.showVertexLabels(true);
       platonic.setGlobalElementColor(Color.RED);

       // The platonic circumsphere
       // Setup globe as a parametrized surface
       PuFunction m_function_c = new PuFunction(2, 3);
       m_function_c.setName("Globe");
       m_function_c.setExpression(0, Double.toString(R) + "*cos(v)*cos(u)");
       m_function_c.setExpression(1, Double.toString(R) + "*cos(v)*sin(u)");
       m_function_c.setExpression(2, Double.toString(R) + "*sin(v)");

       // Setup parametrized surface domain discretization
       PgDomainDescr m_domain_c = new PgDomainDescr(2);
       m_domain_c.setName("Globe Parametrization Domain");
       m_domain_c.setMaxSize(-Math.PI, -Math.PI/2, Math.PI, Math.PI/2);
       m_domain_c.setSize(-Math.PI, -Math.PI/2, Math.PI, Math.PI/2);
       m_domain_c.setDiscr(50, 50);
       m_domain_c.init();

       // Computes parametrized surface
       circumsphere = new PgParmSurface(3);
       circumsphere.setDomainDescr(m_domain_c);
       circumsphere.setFunctionExpr(m_function_c);
       circumsphere.compute();

       // Setup parametrized surface appearance
       circumsphere.setGlobalElementColor(Color.GREEN);
       circumsphere.showEdges(false);
       circumsphere.setTransparency(0.90);
       circumsphere.showTransparency(true);

       // The platonic insphere
       // Setup globe as a parametrized surface
       PuFunction m_function_i = new PuFunction(2, 3);
       m_function_i.setName("Globe");
       m_function_i.setExpression(0, Double.toString(v) + "*cos(v)*cos(u)");
       m_function_i.setExpression(1, Double.toString(v) + "*cos(v)*sin(u)");
       m_function_i.setExpression(2, Double.toString(v) + "*sin(v)");

       // Setup parametrized surface domain discretization
       PgDomainDescr m_domain_i = new PgDomainDescr(2);
       m_domain_i.setName("Globe Parametrization Domain");
       m_domain_i.setMaxSize(-Math.PI, -Math.PI/2, Math.PI, Math.PI/2);
       m_domain_i.setSize(-Math.PI, -Math.PI/2, Math.PI, Math.PI/2);
       m_domain_i.setDiscr(50, 50);
       m_domain_i.init();

       // Computes parametrized surface
       insphere = new PgParmSurface(3);
       insphere.setDomainDescr(m_domain_i);
       insphere.setFunctionExpr(m_function_i);
       insphere.compute();

       // Setup parametrized surface appearance
       insphere.setGlobalElementColor(Color.BLUE);
       insphere.showEdges(false);
       insphere.setTransparency(0.90);
       insphere.showTransparency(true);

       dualvertices = new PgPointSet(3);
       dualvertices.setName("Pontos de tangncia da esfera inscrita");
       dualvertices.setNumVertices(numberOfElements);


       //////////////////////////////////////////////////////////////////////
       // BEGIN OF PART 3 (THE TANGENT POINTS)
       //////////////////////////////////////////////////////////////////////
       // The cube
       if (solidName.equals(cube_trans[idiom]))
       {
          // Tangent points
          tangentpoints[0] = new PdVector(0.0, 0.0, 0.0);
          tangentpoints[1] = new PdVector(0.0, 0.0, 0.0);
          tangentpoints[2] = new PdVector(0.0, 0.0, 0.0);
          tangentpoints[3] = new PdVector(0.0, 0.0, 0.0);
          tangentpoints[4] = new PdVector(0.0, 0.0, 0.0);
          tangentpoints[5] = new PdVector(0.0, 0.0, 0.0);
          double scale = 0.985;
   
          tangentpoints[0].add(vertices[0], vertices[1]);
          tangentpoints[0].add(vertices[2]);
          tangentpoints[0].add(vertices[3]);
          tangentpoints[0].multScalar(1.0/4.0);
          tangentpoints[0].multScalar(scale);
   
          tangentpoints[1].add(vertices[1], vertices[5]);
          tangentpoints[1].add(vertices[6]);
          tangentpoints[1].add(vertices[2]);
          tangentpoints[1].multScalar(1.0/4.0);
          tangentpoints[1].multScalar(scale);
   
          tangentpoints[2].add(vertices[4], vertices[5]);
          tangentpoints[2].add(vertices[6]);
          tangentpoints[2].add(vertices[7]);
          tangentpoints[2].multScalar(1.0/4.0);
          tangentpoints[2].multScalar(scale);
   
          tangentpoints[3].add(vertices[0], vertices[4]);
          tangentpoints[3].add(vertices[7]);
          tangentpoints[3].add(vertices[3]);
          tangentpoints[3].multScalar(1.0/4.0);
          tangentpoints[3].multScalar(scale);
   
          tangentpoints[4].add(vertices[4], vertices[5]);
          tangentpoints[4].add(vertices[1]);
          tangentpoints[4].add(vertices[0]);
          tangentpoints[4].multScalar(1.0/4.0);
          tangentpoints[4].multScalar(scale);
   
          tangentpoints[5].add(vertices[7], vertices[6]);
          tangentpoints[5].add(vertices[2]);
          tangentpoints[5].add(vertices[3]);
          tangentpoints[5].multScalar(1.0/4.0);
          tangentpoints[5].multScalar(scale);
   
          dualvertices.setVertex(0, tangentpoints[0]);
          dualvertices.setVertex(1, tangentpoints[1]);
          dualvertices.setVertex(2, tangentpoints[2]);
          dualvertices.setVertex(3, tangentpoints[3]);
          dualvertices.setVertex(4, tangentpoints[4]);
          dualvertices.setVertex(5, tangentpoints[5]);
       };

       // The regular tetrahedron
       if (solidName.equals(tetrahedron_trans[idiom]))
       {
          tangentpoints[0] = new PdVector(0.0, 0.0, 0.0);
          tangentpoints[1] = new PdVector(0.0, 0.0, 0.0);
          tangentpoints[2] = new PdVector(0.0, 0.0, 0.0);
          tangentpoints[3] = new PdVector(0.0, 0.0, 0.0);
   
          tangentpoints[0].add(vertices[0], vertices[1]);
          tangentpoints[0].add(vertices[3]);
          tangentpoints[0].multScalar(1.0/3.0);
   
          tangentpoints[1].add(vertices[0], vertices[1]);
          tangentpoints[1].add(vertices[2]);
          tangentpoints[1].multScalar(1.0/3.0);
   
          tangentpoints[2].add(vertices[0], vertices[2]);
          tangentpoints[2].add(vertices[3]);
          tangentpoints[2].multScalar(1.0/3.0);
   
          tangentpoints[3].add(vertices[1], vertices[2]);
          tangentpoints[3].add(vertices[3]);
          tangentpoints[3].multScalar(1.0/3.0);
   
          dualvertices.setVertex(0, tangentpoints[0]);
          dualvertices.setVertex(1, tangentpoints[1]);
          dualvertices.setVertex(2, tangentpoints[2]);
          dualvertices.setVertex(3, tangentpoints[3]);
       };

       // The regular octahedron
       if (solidName.equals(octahedron_trans[idiom]))
       {
          // Tangent points
          tangentpoints[0] = new PdVector(0.0, 0.0, 0.0);
          tangentpoints[1] = new PdVector(0.0, 0.0, 0.0);
          tangentpoints[2] = new PdVector(0.0, 0.0, 0.0);
          tangentpoints[3] = new PdVector(0.0, 0.0, 0.0);
          tangentpoints[4] = new PdVector(0.0, 0.0, 0.0);
          tangentpoints[5] = new PdVector(0.0, 0.0, 0.0);
          tangentpoints[6] = new PdVector(0.0, 0.0, 0.0);
          tangentpoints[7] = new PdVector(0.0, 0.0, 0.0);
          double scale = 0.985;
   
          tangentpoints[0].add(vertices[0], vertices[1]);
          tangentpoints[0].add(vertices[4]);
          tangentpoints[0].multScalar(1.0/3.0);
          tangentpoints[0].multScalar(scale);
   
          tangentpoints[1].add(vertices[1], vertices[2]);
          tangentpoints[1].add(vertices[4]);
          tangentpoints[1].multScalar(1.0/3.0);
          tangentpoints[1].multScalar(scale);
   
          tangentpoints[2].add(vertices[2], vertices[3]);
          tangentpoints[2].add(vertices[4]);
          tangentpoints[2].multScalar(1.0/3.0);
          tangentpoints[2].multScalar(scale);
   
          tangentpoints[3].add(vertices[3], vertices[0]);
          tangentpoints[3].add(vertices[4]);
          tangentpoints[3].multScalar(1.0/3.0);
          tangentpoints[3].multScalar(scale);
   
          tangentpoints[4].add(vertices[0], vertices[1]);
          tangentpoints[4].add(vertices[5]);
          tangentpoints[4].multScalar(1.0/3.0);
          tangentpoints[4].multScalar(scale);
   
          tangentpoints[5].add(vertices[1], vertices[2]);
          tangentpoints[5].add(vertices[5]);
          tangentpoints[5].multScalar(1.0/3.0);
          tangentpoints[5].multScalar(scale);

          tangentpoints[6].add(vertices[2], vertices[3]);
          tangentpoints[6].add(vertices[5]);
          tangentpoints[6].multScalar(1.0/3.0);
          tangentpoints[6].multScalar(scale);

          tangentpoints[7].add(vertices[3], vertices[0]);
          tangentpoints[7].add(vertices[5]);
          tangentpoints[7].multScalar(1.0/3.0);
          tangentpoints[7].multScalar(scale);
   
          dualvertices.setVertex(0, tangentpoints[0]);
          dualvertices.setVertex(1, tangentpoints[1]);
          dualvertices.setVertex(2, tangentpoints[2]);
          dualvertices.setVertex(3, tangentpoints[3]);
          dualvertices.setVertex(4, tangentpoints[4]);
          dualvertices.setVertex(5, tangentpoints[5]);
          dualvertices.setVertex(6, tangentpoints[6]);
          dualvertices.setVertex(7, tangentpoints[7]);
       };

       // The regular dodecahedron
       if (solidName.equals(dodecahedron_trans[idiom]))
       {
          // Tangent points
          tangentpoints[0] = new PdVector(0.0, 0.0, 0.0);
          tangentpoints[1] = new PdVector(0.0, 0.0, 0.0);
          tangentpoints[2] = new PdVector(0.0, 0.0, 0.0);
          tangentpoints[3] = new PdVector(0.0, 0.0, 0.0);
          tangentpoints[4] = new PdVector(0.0, 0.0, 0.0);
          tangentpoints[5] = new PdVector(0.0, 0.0, 0.0);
          tangentpoints[6] = new PdVector(0.0, 0.0, 0.0);
          tangentpoints[7] = new PdVector(0.0, 0.0, 0.0);
          tangentpoints[8] = new PdVector(0.0, 0.0, 0.0);
          tangentpoints[9] = new PdVector(0.0, 0.0, 0.0);
          tangentpoints[10] = new PdVector(0.0, 0.0, 0.0);
          tangentpoints[11] = new PdVector(0.0, 0.0, 0.0);
          double scale = 0.985;
   
          tangentpoints[0].add(vertices[0], vertices[1]);
          tangentpoints[0].add(vertices[5]);
          tangentpoints[0].add(vertices[6]);
          tangentpoints[0].add(vertices[18]);
          tangentpoints[0].multScalar(1.0/5.0);
          tangentpoints[0].multScalar(scale);
   
          tangentpoints[1].add(vertices[0], vertices[4]);
          tangentpoints[1].add(vertices[15]);
          tangentpoints[1].add(vertices[18]);
          tangentpoints[1].add(vertices[19]);
          tangentpoints[1].multScalar(1.0/5.0);
          tangentpoints[1].multScalar(scale);
   
          tangentpoints[2].add(vertices[0], vertices[1]);
          tangentpoints[2].add(vertices[2]);
          tangentpoints[2].add(vertices[3]);
          tangentpoints[2].add(vertices[4]);
          tangentpoints[2].multScalar(1.0/5.0);
          tangentpoints[2].multScalar(scale);
   
          tangentpoints[3].add(vertices[5], vertices[9]);
          tangentpoints[3].add(vertices[12]);
          tangentpoints[3].add(vertices[18]);
          tangentpoints[3].add(vertices[19]);
          tangentpoints[3].multScalar(1.0/5.0);
          tangentpoints[3].multScalar(scale);
   
          tangentpoints[4].add(vertices[17], vertices[14]);
          tangentpoints[4].add(vertices[15]);
          tangentpoints[4].add(vertices[4]);
          tangentpoints[4].add(vertices[3]);
          tangentpoints[4].multScalar(1.0/5.0);
          tangentpoints[4].multScalar(scale);
   
          tangentpoints[5].add(vertices[6], vertices[7]);
          tangentpoints[5].add(vertices[11]);
          tangentpoints[5].add(vertices[2]);
          tangentpoints[5].add(vertices[1]);
          tangentpoints[5].multScalar(1.0/5.0);
          tangentpoints[5].multScalar(scale);

          tangentpoints[6].add(vertices[9], vertices[8]);
          tangentpoints[6].add(vertices[7]);
          tangentpoints[6].add(vertices[6]);
          tangentpoints[6].add(vertices[5]);
          tangentpoints[6].multScalar(1.0/5.0);
          tangentpoints[6].multScalar(scale);

          tangentpoints[7].add(vertices[11], vertices[10]);
          tangentpoints[7].add(vertices[17]);
          tangentpoints[7].add(vertices[3]);
          tangentpoints[7].add(vertices[2]);
          tangentpoints[7].multScalar(1.0/5.0);
          tangentpoints[7].multScalar(scale);

          tangentpoints[8].add(vertices[8], vertices[13]);
          tangentpoints[8].add(vertices[10]);
          tangentpoints[8].add(vertices[11]);
          tangentpoints[8].add(vertices[7]);
          tangentpoints[8].multScalar(1.0/5.0);
          tangentpoints[8].multScalar(scale);

          tangentpoints[9].add(vertices[14], vertices[16]);
          tangentpoints[9].add(vertices[12]);
          tangentpoints[9].add(vertices[19]);
          tangentpoints[9].add(vertices[15]);
          tangentpoints[9].multScalar(1.0/5.0);
          tangentpoints[9].multScalar(scale);

          tangentpoints[10].add(vertices[13], vertices[8]);
          tangentpoints[10].add(vertices[9]);
          tangentpoints[10].add(vertices[12]);
          tangentpoints[10].add(vertices[16]);
          tangentpoints[10].multScalar(1.0/5.0);
          tangentpoints[10].multScalar(scale);

          tangentpoints[11].add(vertices[10], vertices[13]);
          tangentpoints[11].add(vertices[16]);
          tangentpoints[11].add(vertices[14]);
          tangentpoints[11].add(vertices[17]);
          tangentpoints[11].multScalar(1.0/5.0);
          tangentpoints[11].multScalar(scale);
   
          dualvertices.setVertex(0, tangentpoints[0]);
          dualvertices.setVertex(1, tangentpoints[1]);
          dualvertices.setVertex(2, tangentpoints[2]);
          dualvertices.setVertex(3, tangentpoints[3]);
          dualvertices.setVertex(4, tangentpoints[4]);
          dualvertices.setVertex(5, tangentpoints[5]);
          dualvertices.setVertex(6, tangentpoints[6]);
          dualvertices.setVertex(7, tangentpoints[7]);
          dualvertices.setVertex(8, tangentpoints[8]);
          dualvertices.setVertex(9, tangentpoints[9]);
          dualvertices.setVertex(10, tangentpoints[10]);
          dualvertices.setVertex(11, tangentpoints[11]);
       };

       // The regular icosahedron
       if (solidName.equals(icosahedron_trans[idiom]))
       {
          // Tangent points
          tangentpoints[0] = new PdVector(0.0, 0.0, 0.0);
          tangentpoints[1] = new PdVector(0.0, 0.0, 0.0);
          tangentpoints[2] = new PdVector(0.0, 0.0, 0.0);
          tangentpoints[3] = new PdVector(0.0, 0.0, 0.0);
          tangentpoints[4] = new PdVector(0.0, 0.0, 0.0);
          tangentpoints[5] = new PdVector(0.0, 0.0, 0.0);
          tangentpoints[6] = new PdVector(0.0, 0.0, 0.0);
          tangentpoints[7] = new PdVector(0.0, 0.0, 0.0);
          tangentpoints[8] = new PdVector(0.0, 0.0, 0.0);
          tangentpoints[9] = new PdVector(0.0, 0.0, 0.0);
          tangentpoints[10] = new PdVector(0.0, 0.0, 0.0);
          tangentpoints[11] = new PdVector(0.0, 0.0, 0.0);
          tangentpoints[12] = new PdVector(0.0, 0.0, 0.0);
          tangentpoints[13] = new PdVector(0.0, 0.0, 0.0);
          tangentpoints[14] = new PdVector(0.0, 0.0, 0.0);
          tangentpoints[15] = new PdVector(0.0, 0.0, 0.0);
          tangentpoints[16] = new PdVector(0.0, 0.0, 0.0);
          tangentpoints[17] = new PdVector(0.0, 0.0, 0.0);
          tangentpoints[18] = new PdVector(0.0, 0.0, 0.0);
          tangentpoints[19] = new PdVector(0.0, 0.0, 0.0);
          double scale = 0.985;
   
          tangentpoints[0].add(vertices[0], vertices[4]);
          tangentpoints[0].add(vertices[8]);
          tangentpoints[0].multScalar(1.0/3.0);
          tangentpoints[0].multScalar(scale);
   
          tangentpoints[1].add(vertices[8], vertices[0]);
          tangentpoints[1].add(vertices[3]);
          tangentpoints[1].multScalar(1.0/3.0);
          tangentpoints[1].multScalar(scale);
   
          tangentpoints[2].add(vertices[0], vertices[9]);
          tangentpoints[2].add(vertices[3]);
          tangentpoints[2].multScalar(1.0/3.0);
          tangentpoints[2].multScalar(scale);
   
          tangentpoints[3].add(vertices[4], vertices[0]);
          tangentpoints[3].add(vertices[7]);
          tangentpoints[3].multScalar(1.0/3.0);
          tangentpoints[3].multScalar(scale);
   
          tangentpoints[4].add(vertices[7], vertices[9]);
          tangentpoints[4].add(vertices[0]);
          tangentpoints[4].multScalar(1.0/3.0);
          tangentpoints[4].multScalar(scale);
   
          tangentpoints[5].add(vertices[7], vertices[10]);
          tangentpoints[5].add(vertices[9]);
          tangentpoints[5].multScalar(1.0/3.0);
          tangentpoints[5].multScalar(scale);

          tangentpoints[6].add(vertices[10], vertices[6]);
          tangentpoints[6].add(vertices[9]);
          tangentpoints[6].multScalar(1.0/3.0);
          tangentpoints[6].multScalar(scale);

          tangentpoints[7].add(vertices[6], vertices[3]);
          tangentpoints[7].add(vertices[9]);
          tangentpoints[7].multScalar(1.0/3.0);
          tangentpoints[7].multScalar(scale);

          tangentpoints[8].add(vertices[6], vertices[5]);
          tangentpoints[8].add(vertices[3]);
          tangentpoints[8].multScalar(1.0/3.0);
          tangentpoints[8].multScalar(scale);

          tangentpoints[9].add(vertices[3], vertices[5]);
          tangentpoints[9].add(vertices[8]);
          tangentpoints[9].multScalar(1.0/3.0);
          tangentpoints[9].multScalar(scale);

          tangentpoints[10].add(vertices[5], vertices[11]);
          tangentpoints[10].add(vertices[8]);
          tangentpoints[10].multScalar(1.0/3.0);
          tangentpoints[10].multScalar(scale);

          tangentpoints[11].add(vertices[8], vertices[11]);
          tangentpoints[11].add(vertices[4]);
          tangentpoints[11].multScalar(1.0/3.0);
          tangentpoints[11].multScalar(scale);

          tangentpoints[12].add(vertices[11], vertices[4]);
          tangentpoints[12].add(vertices[1]);
          tangentpoints[12].multScalar(1.0/3.0);
          tangentpoints[12].multScalar(scale);

          tangentpoints[13].add(vertices[4], vertices[1]);
          tangentpoints[13].add(vertices[7]);
          tangentpoints[13].multScalar(1.0/3.0);
          tangentpoints[13].multScalar(scale);

          tangentpoints[14].add(vertices[1], vertices[10]);
          tangentpoints[14].add(vertices[7]);
          tangentpoints[14].multScalar(1.0/3.0);
          tangentpoints[14].multScalar(scale);

          tangentpoints[15].add(vertices[10], vertices[2]);
          tangentpoints[15].add(vertices[6]);
          tangentpoints[15].multScalar(1.0/3.0);
          tangentpoints[15].multScalar(scale);

          tangentpoints[16].add(vertices[6], vertices[2]);
          tangentpoints[16].add(vertices[5]);
          tangentpoints[16].multScalar(1.0/3.0);
          tangentpoints[16].multScalar(scale);

          tangentpoints[17].add(vertices[2], vertices[5]);
          tangentpoints[17].add(vertices[11]);
          tangentpoints[17].multScalar(1.0/3.0);
          tangentpoints[17].multScalar(scale);

          tangentpoints[18].add(vertices[10], vertices[1]);
          tangentpoints[18].add(vertices[2]);
          tangentpoints[18].multScalar(1.0/3.0);
          tangentpoints[18].multScalar(scale);

          tangentpoints[19].add(vertices[1], vertices[2]);
          tangentpoints[19].add(vertices[11]);
          tangentpoints[19].multScalar(1.0/3.0);
          tangentpoints[19].multScalar(scale);

          dualvertices.setVertex(0, tangentpoints[0]);
          dualvertices.setVertex(1, tangentpoints[1]);
          dualvertices.setVertex(2, tangentpoints[2]);
          dualvertices.setVertex(3, tangentpoints[3]);
          dualvertices.setVertex(4, tangentpoints[4]);
          dualvertices.setVertex(5, tangentpoints[5]);
          dualvertices.setVertex(6, tangentpoints[6]);
          dualvertices.setVertex(7, tangentpoints[7]);
          dualvertices.setVertex(8, tangentpoints[8]);
          dualvertices.setVertex(9, tangentpoints[9]);
          dualvertices.setVertex(10, tangentpoints[10]);
          dualvertices.setVertex(11, tangentpoints[11]);
          dualvertices.setVertex(12, tangentpoints[12]);
          dualvertices.setVertex(13, tangentpoints[13]);
          dualvertices.setVertex(14, tangentpoints[14]);
          dualvertices.setVertex(15, tangentpoints[15]);
          dualvertices.setVertex(16, tangentpoints[16]);
          dualvertices.setVertex(17, tangentpoints[17]);
          dualvertices.setVertex(18, tangentpoints[18]);
          dualvertices.setVertex(19, tangentpoints[19]);
       };
       //////////////////////////////////////////////////////////////////////
       // END OF PART 3 
       //////////////////////////////////////////////////////////////////////
       

       dualvertices.setGlobalVertexColor(Color.YELLOW);
       dualvertices.setGlobalVertexSize(3.0);
       dualvertices.showVertexSizes(true);

       // Dual
       dual = new PgElementSet(3); // This is a 3D object
       dual.setNumElements(numberOfVertices); 
       dual.setNumVertices(numberOfElements);


       //////////////////////////////////////////////////////////////////////
       // BEGIN OF PART 4 (THE DUAL)
       //////////////////////////////////////////////////////////////////////
       // The cube
       if (solidName.equals(cube_trans[idiom]))
       {
          // Defining the vertices coordinates
          dual.setVertex(0, tangentpoints[0]);
          dual.setVertex(1, tangentpoints[1]);
          dual.setVertex(2, tangentpoints[2]);
          dual.setVertex(3, tangentpoints[3]);
          dual.setVertex(4, tangentpoints[4]);
          dual.setVertex(5, tangentpoints[5]);
   
          // Setup faces specifying vertices id order
          dual.setElement(0, 0, 4, 1);
          dual.setElement(1, 0, 1, 5);
          dual.setElement(2, 0, 5, 3);
          dual.setElement(3, 0, 3, 4);
          dual.setElement(4, 2, 1, 4);
          dual.setElement(5, 2, 5, 1);
          dual.setElement(6, 2, 3, 5);
          dual.setElement(7, 2, 4, 3);
       };

       // The tetrahedron
       if (solidName.equals(tetrahedron_trans[idiom]))
       {
          // Defining the vertices coordinates
          dual.setVertex(0, tangentpoints[0]);
          dual.setVertex(1, tangentpoints[1]);
          dual.setVertex(2, tangentpoints[2]);
          dual.setVertex(3, tangentpoints[3]);

          // Setup faces specifying vertices id order
          dual.setElement(0, 0, 1, 2);
          dual.setElement(1, 0, 1, 3);
          dual.setElement(2, 1, 2, 3);
          dual.setElement(3, 2, 0, 3);
       };

       // The octahedron
       if (solidName.equals(octahedron_trans[idiom]))
       {
          // Defining the vertices coordinates
          dual.setVertex(0, tangentpoints[0]);
          dual.setVertex(1, tangentpoints[1]);
          dual.setVertex(2, tangentpoints[2]);
          dual.setVertex(3, tangentpoints[3]);
          dual.setVertex(4, tangentpoints[4]);
          dual.setVertex(5, tangentpoints[5]);
          dual.setVertex(6, tangentpoints[6]);
          dual.setVertex(7, tangentpoints[7]);
   
          // Setup faces specifying vertices id order
          dual.setElement(0, 0, 1, 2, 3);
          dual.setElement(1, 0, 4, 5, 1);
          dual.setElement(2, 1, 5, 6, 2);
          dual.setElement(3, 6, 2, 3, 7);
          dual.setElement(4, 3, 7, 4, 0);
          dual.setElement(5, 4, 5, 6, 7);
       };

       // The regular dodecahedron
       if (solidName.equals(dodecahedron_trans[idiom]))
       {
          // Defining the vertices coordinates
          dual.setVertex(0, tangentpoints[0]);
          dual.setVertex(1, tangentpoints[1]);
          dual.setVertex(2, tangentpoints[2]);
          dual.setVertex(3, tangentpoints[3]);
          dual.setVertex(4, tangentpoints[4]);
          dual.setVertex(5, tangentpoints[5]);
          dual.setVertex(6, tangentpoints[6]);
          dual.setVertex(7, tangentpoints[7]);
          dual.setVertex(8, tangentpoints[8]);
          dual.setVertex(9, tangentpoints[9]);
          dual.setVertex(10, tangentpoints[10]);
          dual.setVertex(11, tangentpoints[11]);
   
          // Setup faces specifying vertices id order
          dual.setElement(0, 0, 1, 2);
          dual.setElement(1, 2, 1, 4);
          dual.setElement(2, 2, 4, 7);
          dual.setElement(3, 4, 7, 11);
          dual.setElement(4, 7, 11, 8);
          dual.setElement(5, 11, 8, 10);
          dual.setElement(6, 6, 8, 10);
          dual.setElement(7, 7, 8, 5);
          dual.setElement(8, 2, 7, 5);
          dual.setElement(9, 2, 5, 0);
          dual.setElement(10, 8, 6, 5);
          dual.setElement(11, 5, 0, 6);
          dual.setElement(12, 0, 6, 3);
          dual.setElement(13, 6, 3, 10);
          dual.setElement(14, 0, 1, 3);
          dual.setElement(15, 3, 10, 9);
          dual.setElement(16, 9, 10, 11);
          dual.setElement(17, 9, 3, 1);
          dual.setElement(18, 9, 1, 4);
          dual.setElement(19, 9, 4, 11);
       };

       // The regular icosahedron
       if (solidName.equals(icosahedron_trans[idiom]))
       {
          // Defining the vertices coordinates
          dual.setVertex(0,  tangentpoints[0]);
          dual.setVertex(1,  tangentpoints[1]);
          dual.setVertex(2,  tangentpoints[2]);
          dual.setVertex(3,  tangentpoints[3]);
          dual.setVertex(4,  tangentpoints[4]);
          dual.setVertex(5,  tangentpoints[5]);
          dual.setVertex(6,  tangentpoints[6]);
          dual.setVertex(7,  tangentpoints[7]);
          dual.setVertex(8,  tangentpoints[8]);
          dual.setVertex(9,  tangentpoints[9]);
          dual.setVertex(10, tangentpoints[10]);
          dual.setVertex(11, tangentpoints[11]);
          dual.setVertex(12, tangentpoints[12]);
          dual.setVertex(13, tangentpoints[13]);
          dual.setVertex(14, tangentpoints[14]);
          dual.setVertex(15, tangentpoints[15]);
          dual.setVertex(16, tangentpoints[16]);
          dual.setVertex(17, tangentpoints[17]);
          dual.setVertex(18, tangentpoints[18]);
          dual.setVertex(19, tangentpoints[19]);
            
          // Setup faces specifying vertices id order
          PiVector idxd00 = new PiVector(5);
          idxd00.setEntry(0, 1);
          idxd00.setEntry(1, 9);
          idxd00.setEntry(2, 10);
          idxd00.setEntry(3, 11);
          idxd00.setEntry(4, 0);
          dual.setElement(0, idxd00);
              
          PiVector idxd01 = new PiVector(5);
          idxd01.setEntry(0, 1);
          idxd01.setEntry(1, 0);
          idxd01.setEntry(2, 3);
          idxd01.setEntry(3, 4);
          idxd01.setEntry(4, 2);
          dual.setElement(1, idxd01);         
   
          PiVector idxd02 = new PiVector(5);
          idxd02.setEntry(0, 0);
          idxd02.setEntry(1, 11);
          idxd02.setEntry(2, 12);
          idxd02.setEntry(3, 13);
          idxd02.setEntry(4, 3);
          dual.setElement(2, idxd02);          
   
          PiVector idxd03 = new PiVector(5);
          idxd03.setEntry(0, 1);
          idxd03.setEntry(1, 2);
          idxd03.setEntry(2, 7);
          idxd03.setEntry(3, 8);
          idxd03.setEntry(4, 9);
          dual.setElement(3, idxd03);           
              
          PiVector idxd04 = new PiVector(5);
          idxd04.setEntry(0, 3);
          idxd04.setEntry(1, 13);
          idxd04.setEntry(2, 14);
          idxd04.setEntry(3, 5);
          idxd04.setEntry(4, 4);
          dual.setElement(4, idxd04);
              
          PiVector idxd05 = new PiVector(5);
          idxd05.setEntry(0, 11);
          idxd05.setEntry(1, 10);
          idxd05.setEntry(2, 17);
          idxd05.setEntry(3, 19);
          idxd05.setEntry(4, 12);
          dual.setElement(5, idxd05);
              
          PiVector idxd06 = new PiVector(5);
          idxd06.setEntry(0, 17);
          idxd06.setEntry(1, 10);
          idxd06.setEntry(2, 9);
          idxd06.setEntry(3, 8);
          idxd06.setEntry(4, 16);
          dual.setElement(6, idxd06);
              
          PiVector idxd07 = new PiVector(5);
          idxd07.setEntry(0, 16);
          idxd07.setEntry(1, 15);
          idxd07.setEntry(2, 18);
          idxd07.setEntry(3, 19);
          idxd07.setEntry(4, 17);
          dual.setElement(7, idxd07);
              
          PiVector idxd08 = new PiVector(5);
          idxd08.setEntry(0, 18);
          idxd08.setEntry(1, 14);
          idxd08.setEntry(2, 13);
          idxd08.setEntry(3, 12);
          idxd08.setEntry(4, 19);
          dual.setElement(8, idxd08);
              
          PiVector idxd09 = new PiVector(5);
          idxd09.setEntry(0, 6);
          idxd09.setEntry(1, 5);
          idxd09.setEntry(2, 14);
          idxd09.setEntry(3, 18);
          idxd09.setEntry(4, 15);
          dual.setElement(9, idxd09);
              
          PiVector idxd10 = new PiVector(5);
          idxd10.setEntry(0, 15);
          idxd10.setEntry(1, 16);
          idxd10.setEntry(2, 8);
          idxd10.setEntry(3, 7);
          idxd10.setEntry(4, 6);
          dual.setElement(10, idxd10);
              
          PiVector idxd11 = new PiVector(5);
          idxd11.setEntry(0, 4);
          idxd11.setEntry(1, 5);
          idxd11.setEntry(2, 6);
          idxd11.setEntry(3, 7);
          idxd11.setEntry(4, 2);
          dual.setElement(11, idxd11);
       };
       //////////////////////////////////////////////////////////////////////
       // END OF PART 4 
       //////////////////////////////////////////////////////////////////////
       

       // Configuring the dual
       dual.setTransparency(0.5);
       dual.showVertices(true);
       dual.showTransparency(true);
       dual.showElementColors(true);
       dual.showElements(true);
       dual.showEdges(true);
       dual.showVertexOutline(true);
       dual.showVertexOutline(true);
       dual.setGlobalElementColor(Color.YELLOW);


       //////////////////////////////////////////////////////////////////////
       // BEGIN OF PART 5 (THE SPECIAL OBJECTS)
       //////////////////////////////////////////////////////////////////////
       // The cube
       if (solidName.equals(cube_trans[idiom]))
       {
          // Tetrahedron
          tetrahedron_ic = new PgElementSet(3); // This is a 3D object
   
          tetrahedron_ic.setNumElements(4); // Our platonic has 6 faces
          tetrahedron_ic.setNumVertices(4); // Our platonic has 8 vertices
   
          // Defining the vertices coordinates
          tetrahedron_ic.setVertex(0, vertices[0]);
          tetrahedron_ic.setVertex(1, vertices[2]);
          tetrahedron_ic.setVertex(2, vertices[5]);
          tetrahedron_ic.setVertex(3, vertices[7]);
   
          // Setup faces specifying vertices id order
          tetrahedron_ic.setElement(0, 0, 3, 2);
          tetrahedron_ic.setElement(1, 0, 2, 1);
          tetrahedron_ic.setElement(2, 3, 0, 1);
          tetrahedron_ic.setElement(3, 1, 2, 3);
   
          // Configuring the solid
          tetrahedron_ic.setTransparency(0.5);
          tetrahedron_ic.showVertices(true);
          tetrahedron_ic.showTransparency(true);
          tetrahedron_ic.showElementColors(true);
          tetrahedron_ic.showElements(true);
          tetrahedron_ic.showEdges(true);
          tetrahedron_ic.showVertexOutline(true);
          tetrahedron_ic.showVertexOutline(true);
          tetrahedron_ic.setGlobalElementColor(Color.CYAN);   
       };

       // The regular octahedron
       if (solidName.equals(octahedron_trans[idiom]))
       {
          double w = (Math.sqrt(5.0) - 1.0)/2.0;

          // Regular icosahedron inscribed in the octahedron
          icosahedron_io = new PgElementSet(3); // This is a 3D object
   
          icosahedron_io.setNumElements(20); // Our platonic has 20 faces
          icosahedron_io.setNumVertices(12); // Our platonic has 12 vertices

          // Setup vertices coordinates
          PdVector G = new PdVector( R-R*w,  R*w,  0);
          PdVector H = new PdVector( R*w-R,  R*w,  0);
          PdVector I = new PdVector( R*w-R, -R*w,  0);
          PdVector J = new PdVector( R-R*w, -R*w,  0);
          PdVector L = new PdVector( 0, R-R*w,  R*w);
          PdVector M = new PdVector( 0, R*w-R,  R*w);
          PdVector N = new PdVector( 0, R*w-R, -R*w);
          PdVector O = new PdVector( 0, R-R*w, -R*w);
          PdVector P = new PdVector(  R*w,  0,  R-R*w);
          PdVector Q = new PdVector(  R*w,  0,  R*w-R);
          PdVector S = new PdVector( -R*w,  0,  R*w-R);
          PdVector T = new PdVector( -R*w,  0,  R-R*w);

          double scale = 0.985;
          G.multScalar(scale);
          H.multScalar(scale);
          I.multScalar(scale);
          J.multScalar(scale);
          L.multScalar(scale);
          M.multScalar(scale);
          N.multScalar(scale);
          O.multScalar(scale);
          P.multScalar(scale);
          Q.multScalar(scale);
          S.multScalar(scale);
          T.multScalar(scale);

          // Defining the vertices coordinates
          icosahedron_io.setVertex(0, G);
          icosahedron_io.setVertex(1, H);
          icosahedron_io.setVertex(2, I);
          icosahedron_io.setVertex(3, J);
          icosahedron_io.setVertex(4, L);
          icosahedron_io.setVertex(5, M);
          icosahedron_io.setVertex(6, N);
          icosahedron_io.setVertex(7, O);
          icosahedron_io.setVertex(8, P);
          icosahedron_io.setVertex(9, Q);
          icosahedron_io.setVertex(10, S);
          icosahedron_io.setVertex(11, T);

          // Defining the vertices names
          icosahedron_io.getVertex(0).setName("G");
          icosahedron_io.getVertex(1).setName("H");
          icosahedron_io.getVertex(2).setName("I");
          icosahedron_io.getVertex(3).setName("J");
          icosahedron_io.getVertex(4).setName("L");
          icosahedron_io.getVertex(5).setName("M");
          icosahedron_io.getVertex(6).setName("N");
          icosahedron_io.getVertex(7).setName("O");
          icosahedron_io.getVertex(8).setName("P");
          icosahedron_io.getVertex(9).setName("Q");
          icosahedron_io.getVertex(10).setName("S");
          icosahedron_io.getVertex(11).setName("T");

          // Setup faces specifying vertices id order
          icosahedron_io.setElement(0,  0,  1,  4);
          icosahedron_io.setElement(1,  0,  7,  1);
          icosahedron_io.setElement(2,  1,  7, 10);
          icosahedron_io.setElement(3,  1, 10, 11);
          icosahedron_io.setElement(4,  1, 11,  4);
          icosahedron_io.setElement(5,  0,  4,  8);
          icosahedron_io.setElement(6,  0,  8,  9);
          icosahedron_io.setElement(7,  0,  9,  7);
          icosahedron_io.setElement(8,  8,  9,  3);
          icosahedron_io.setElement(9,  8,  3,  5);
          icosahedron_io.setElement(10, 8,  5,  4);
          icosahedron_io.setElement(11, 5,  4, 11);
          icosahedron_io.setElement(12, 5, 11,  2);
          icosahedron_io.setElement(13, 5,  2,  3);
          icosahedron_io.setElement(14, 7,  9,  6);
          icosahedron_io.setElement(15, 7,  6, 10);
          icosahedron_io.setElement(16, 6,  10, 2);
          icosahedron_io.setElement(17, 6,  2,  3);
          icosahedron_io.setElement(18, 2, 11, 10);
          icosahedron_io.setElement(19, 9,  6,  3);
            

          // Configuring the solid
          icosahedron_io.setTransparency(0.5);
          icosahedron_io.showVertices(true);
          icosahedron_io.showTransparency(true);
          icosahedron_io.showElementColors(true);
          icosahedron_io.showElements(true);
          icosahedron_io.showEdges(true);
          icosahedron_io.showVertexOutline(true);
          icosahedron_io.showVertexOutline(true);
          icosahedron_io.setGlobalElementColor(Color.CYAN);   
       };

       // The regular dodecahedron
       if (solidName.equals(dodecahedron_trans[idiom]))
       {
          // goldenrectangle1
          goldenrectangle1 = new PgElementSet(3); // This is a 3D object

          goldenrectangle1.setNumElements(1); // Our icosaedro has 1 faces
          goldenrectangle1.setNumVertices(4); // Our icosaedro has 4 vertices
         
          // Defining the vertices coordinates
          goldenrectangle1.setVertex(0,  tangentpoints[4]);
          goldenrectangle1.setVertex(1,  tangentpoints[10]);
          goldenrectangle1.setVertex(2,  tangentpoints[6]);
          goldenrectangle1.setVertex(3,  tangentpoints[2]);
       
          // Setup faces specifying vertices id order
          PiVector idxc100 = new PiVector(4);
          idxc100.setEntry(0, 0);
          idxc100.setEntry(1, 1);
          idxc100.setEntry(2, 2);
          idxc100.setEntry(3, 3);
          goldenrectangle1.setElement(0, idxc100);
        

          // Configuring the solid
          goldenrectangle1.setTransparency(0.5);
          goldenrectangle1.showVertices(true);
          goldenrectangle1.showTransparency(true);
          goldenrectangle1.showElementColors(true);
          goldenrectangle1.showElements(true);
          goldenrectangle1.showEdges(true);
          goldenrectangle1.showVertexOutline(true);
          goldenrectangle1.showVertexOutline(true);
          goldenrectangle1.setGlobalElementColor(new Color(192, 103, 126));

          // goldenrectangle2
          goldenrectangle2 = new PgElementSet(3); // This is a 3D object

          goldenrectangle2.setNumElements(1); // Our icosaedro has 1 faces
          goldenrectangle2.setNumVertices(4); // Our icosaedro has 4 vertices

          // Defining the vertices coordinates
          goldenrectangle2.setVertex(0,  tangentpoints[0]);
          goldenrectangle2.setVertex(1,  tangentpoints[5]);
          goldenrectangle2.setVertex(2,  tangentpoints[11]);
          goldenrectangle2.setVertex(3,  tangentpoints[9]);
       
          // Setup faces specifying vertices id order
          PiVector idxc200 = new PiVector(4);
          idxc200.setEntry(0, 0);
          idxc200.setEntry(1, 1);
          idxc200.setEntry(2, 2);
          idxc200.setEntry(3, 3);
          goldenrectangle2.setElement(0, idxc200);
        

          // Configuring the solid
          goldenrectangle2.setTransparency(0.5);
          goldenrectangle2.showVertices(true);
          goldenrectangle2.showTransparency(true);
          goldenrectangle2.showElementColors(true);
          goldenrectangle2.showElements(true);
          goldenrectangle2.showEdges(true);
          goldenrectangle2.showVertexOutline(true);
          goldenrectangle2.showVertexOutline(true);
          goldenrectangle2.setGlobalElementColor(Color.blue);
           
          // goldenrectangle3
          goldenrectangle3 = new PgElementSet(3); // This is a 3D object

          goldenrectangle3.setNumElements(1); // Our icosaedro has 1 faces
          goldenrectangle3.setNumVertices(4); // Our icosaedro has 4 vertices

          // Defining the vertices coordinates
          goldenrectangle3.setVertex(0,  tangentpoints[8]);
          goldenrectangle3.setVertex(1,  tangentpoints[7]);
          goldenrectangle3.setVertex(2,  tangentpoints[1]);
          goldenrectangle3.setVertex(3,  tangentpoints[3]);
	  
          // Setup faces specifying vertices id order
          PiVector idxc300 = new PiVector(4);
          idxc300.setEntry(0, 0);
          idxc300.setEntry(1, 1);
          idxc300.setEntry(2, 2);
          idxc300.setEntry(3, 3);
          goldenrectangle3.setElement(0, idxc300);

          // Configuring the solid
          goldenrectangle3.setTransparency(0.5);
          goldenrectangle3.showVertices(true);
          goldenrectangle3.showTransparency(true);
          goldenrectangle3.showElementColors(true);
          goldenrectangle3.showElements(true);
          goldenrectangle3.showEdges(true);
          goldenrectangle3.showVertexOutline(true);
          goldenrectangle3.showVertexOutline(true);
          goldenrectangle3.setGlobalElementColor(new Color(255, 144, 0));
       

          //cube_id
          cube_id = new PgElementSet(3); // This is a 3D object

          cube_id.setNumElements(6); // Our cube_id has 6 faces
          cube_id.setNumVertices(8); // Our Cube_id has 8 vertices

          // Defining the vertices coordinates
          cube_id.setVertex(0,  vertices[5]);
          cube_id.setVertex(1,  vertices[7]);
          cube_id.setVertex(2,  vertices[2]);
          cube_id.setVertex(3,  vertices[0]);
          cube_id.setVertex(4,  vertices[12]);
          cube_id.setVertex(5,  vertices[13]);
          cube_id.setVertex(6,  vertices[17]);
          cube_id.setVertex(7,  vertices[15]);

          // Setup faces specifying vertices id order
          PiVector idxc00 = new PiVector(4);
          idxc00.setEntry(0, 0);
          idxc00.setEntry(1, 1);
          idxc00.setEntry(2, 2);
          idxc00.setEntry(3, 3);
          cube_id.setElement(0, idxc00);

          PiVector idxc01 = new PiVector(4);
          idxc01.setEntry(0, 4);
          idxc01.setEntry(1, 5);
          idxc01.setEntry(2, 6);
          idxc01.setEntry(3, 7);
          cube_id.setElement(1, idxc01);

          PiVector idxc02 = new PiVector(4);
          idxc02.setEntry(0, 4);
          idxc02.setEntry(1, 0);
          idxc02.setEntry(2, 3);
          idxc02.setEntry(3, 7);
          cube_id.setElement(2, idxc02);

          PiVector idxc03 = new PiVector(4);
          idxc03.setEntry(0, 5);
          idxc03.setEntry(1, 1);
          idxc03.setEntry(2, 2);
          idxc03.setEntry(3, 6);
          cube_id.setElement(3, idxc03);

          PiVector idxc04 = new PiVector(4);
          idxc04.setEntry(0, 4);
          idxc04.setEntry(1, 0);
          idxc04.setEntry(2, 1);
          idxc04.setEntry(3, 5);
          cube_id.setElement(4, idxc04);

          PiVector idxc05 = new PiVector(4);
          idxc05.setEntry(0, 7);
          idxc05.setEntry(1, 3);
          idxc05.setEntry(2, 2);
          idxc05.setEntry(3, 6);
          cube_id.setElement(5, idxc05);

          // Configuring the solid
          cube_id.setTransparency(0.5);
          cube_id.showVertices(true);
          cube_id.showTransparency(true);
          cube_id.showElementColors(true);
          cube_id.showElements(true);
          cube_id.showEdges(true);
          cube_id.showVertexOutline(true);
          cube_id.showVertexOutline(true);
          cube_id.setGlobalElementColor(Color.cyan);

          //tetrahedron_id
          tetrahedron_id = new PgElementSet(3); // This is a 3D object

          tetrahedron_id.setNumElements(4); // Our tetraheron_id has 4 faces
          tetrahedron_id.setNumVertices(4); // Our tetrahedron_id has 4 vertices

          // Defining the vertices coordinates
          tetrahedron_id.setVertex(0,  vertices[17]);
          tetrahedron_id.setVertex(1,  vertices[12]);
          tetrahedron_id.setVertex(2,  vertices[7]);
          tetrahedron_id.setVertex(3,  vertices[0]);

          // Setup faces specifying vertices id order
          PiVector idxt00 = new PiVector(3);
          idxt00.setEntry(0, 0);
          idxt00.setEntry(1, 1);
          idxt00.setEntry(2, 2);
          tetrahedron_id.setElement(0, idxt00);

          PiVector idxt01 = new PiVector(3);
          idxt01.setEntry(0, 0);
          idxt01.setEntry(1, 3);
          idxt01.setEntry(2, 1);
          tetrahedron_id.setElement(1, idxt01);

          PiVector idxt02 = new PiVector(3);
          idxt02.setEntry(0, 1);
          idxt02.setEntry(1, 3);
          idxt02.setEntry(2, 2);
          tetrahedron_id.setElement(2, idxt02);

          PiVector idxt03 = new PiVector(3);
          idxt03.setEntry(0, 2);
          idxt03.setEntry(1, 3);
          idxt03.setEntry(2, 0);
          tetrahedron_id.setElement(3, idxt03);

          // Configuring the solid
          tetrahedron_id.setTransparency(0.5);
          tetrahedron_id.showVertices(true);
          tetrahedron_id.showTransparency(true);
          tetrahedron_id.showElementColors(true);
          tetrahedron_id.showElements(true);
          tetrahedron_id.showEdges(true);
          tetrahedron_id.showVertexOutline(true);
          tetrahedron_id.showVertexOutline(true);
          tetrahedron_id.setGlobalElementColor(Color.green);
		  
       };

       // The regular icosahedron
       if (solidName.equals(icosahedron_trans[idiom]))
       {
          // goldenrectangle1
          goldenrectangle1 = new PgElementSet(3); // This is a 3D object

          goldenrectangle1.setNumElements(1); // Our icosaedro has 1 faces
          goldenrectangle1.setNumVertices(4); // Our icosaedro has 4 vertices
         
          // Defining the vertices coordinates
          goldenrectangle1.setVertex(0,  vertices[11]);
          goldenrectangle1.setVertex(1,  vertices[4]);
          goldenrectangle1.setVertex(2,  vertices[9]);
          goldenrectangle1.setVertex(3,  vertices[6]);
       
          // Setup faces specifying vertices id order
          PiVector idxc100 = new PiVector(4);
          idxc100.setEntry(0, 0);
          idxc100.setEntry(1, 1);
          idxc100.setEntry(2, 2);
          idxc100.setEntry(3, 3);
          goldenrectangle1.setElement(0, idxc100);
        

          // Configuring the solid
          goldenrectangle1.setTransparency(0.5);
          goldenrectangle1.showVertices(true);
          goldenrectangle1.showTransparency(true);
          goldenrectangle1.showElementColors(true);
          goldenrectangle1.showElements(true);
          goldenrectangle1.showEdges(true);
          goldenrectangle1.showVertexOutline(true);
          goldenrectangle1.showVertexOutline(true);
          goldenrectangle1.setGlobalElementColor(new Color(192, 103, 126));

          // goldenrectangle2
          goldenrectangle2 = new PgElementSet(3); // This is a 3D object

          goldenrectangle2.setNumElements(1); // Our icosaedro has 1 faces
          goldenrectangle2.setNumVertices(4); // Our icosaedro has 4 vertices

          // Defining the vertices coordinates
          goldenrectangle2.setVertex(0,  vertices[1]);
          goldenrectangle2.setVertex(1,  vertices[10]);
          goldenrectangle2.setVertex(2,  vertices[3]);
          goldenrectangle2.setVertex(3,  vertices[8]);
       
          // Setup faces specifying vertices id order
          PiVector idxc200 = new PiVector(4);
          idxc200.setEntry(0, 0);
          idxc200.setEntry(1, 1);
          idxc200.setEntry(2, 2);
          idxc200.setEntry(3, 3);
          goldenrectangle2.setElement(0, idxc200);
        

          // Configuring the solid
          goldenrectangle2.setTransparency(0.5);
          goldenrectangle2.showVertices(true);
          goldenrectangle2.showTransparency(true);
          goldenrectangle2.showElementColors(true);
          goldenrectangle2.showElements(true);
          goldenrectangle2.showEdges(true);
          goldenrectangle2.showVertexOutline(true);
          goldenrectangle2.showVertexOutline(true);
          goldenrectangle2.setGlobalElementColor(Color.blue);
           
           // goldenrectangle3
          goldenrectangle3 = new PgElementSet(3); // This is a 3D object

          goldenrectangle3.setNumElements(1); // Our icosaedro has 1 faces
          goldenrectangle3.setNumVertices(4); // Our icosaedro has 4 vertices


         // Defining the vertices coordinates
          goldenrectangle3.setVertex(0,  vertices[2]);
          goldenrectangle3.setVertex(1,  vertices[5]);
          goldenrectangle3.setVertex(2,  vertices[0]);
          goldenrectangle3.setVertex(3,  vertices[7]);
       
          // Setup faces specifying vertices id order
          PiVector idxc300 = new PiVector(4);
          idxc300.setEntry(0, 0);
          idxc300.setEntry(1, 1);
          idxc300.setEntry(2, 2);
          idxc300.setEntry(3, 3);
          goldenrectangle3.setElement(0, idxc300);

          // Configuring the solid
          goldenrectangle3.setTransparency(0.5);
          goldenrectangle3.showVertices(true);
          goldenrectangle3.showTransparency(true);
          goldenrectangle3.showElementColors(true);
          goldenrectangle3.showElements(true);
          goldenrectangle3.showEdges(true);
          goldenrectangle3.showVertexOutline(true);
          goldenrectangle3.showVertexOutline(true);
          goldenrectangle3.setGlobalElementColor(new Color(255, 144, 0));
       };
       //////////////////////////////////////////////////////////////////////
       // END OF PART 5 
       //////////////////////////////////////////////////////////////////////
       

       // Creates the picked point
       picked_point = new PgPolygon(3);
       picked_point.setName("Picked Point");
       picked_point.setNumVertices(2);
       picked_point.showVertices(true);
       picked_point.setVertexColor(1, Color.BLACK);
       picked_point.showVertexColors(true);
       picked_point.setGlobalEdgeColor(Color.BLACK);
       picked_point.setVertex(0, 0.0, 0.0, 0.0);
       double lat_in_rad = 90.0*Math.PI/180.0;
       double lon_in_rad = 90.0*Math.PI/180.0;
       double x = R*Math.cos(lat_in_rad)*Math.cos(lon_in_rad);
       double y = R*Math.cos(lat_in_rad)*Math.sin(lon_in_rad);
       double z = R*Math.sin(lat_in_rad);
       picked_point.setVertex(1, x, y, z);
       picked_point.getVertex(0).setName("");
       picked_point.getVertex(1).setName("e");
       picked_point.setLabelColor(0, Color.BLACK);
       picked_point.setLabelColor(1, Color.BLACK);
       picked_point.setLabelSize(0, 20);
       picked_point.setLabelSize(1, 20);
       picked_point.showVertexLabels(true);
       picked_point.showPolygonEndArrow(true);

       // Rotation axis
       PdVector v1 = new PdVector(0, 0, 1);
       PdVector v2 = new PdVector(picked_point.getVertex(1).getEntry(0),
                                  picked_point.getVertex(1).getEntry(1),
                                  picked_point.getVertex(1).getEntry(2));
       rotationAxis = v1.crossNew(v1, v2);
       rotationAxis.normalize();

       // Cut plane
       cutplane = new PgElementSet(3); // This is a 3D object
       cutplane.setNumElements(1);
       cutplane.setNumVertices(4); // Our platonic has 8 vertices
       cutplane.setVertex(0,  R,  R, 0);
       cutplane.setVertex(1, -R,  R, 0);
       cutplane.setVertex(2, -R, -R, 0);
       cutplane.setVertex(3,  R, -R, 0);
       cutplane.setElement(0, 0, 1, 2, 3);
       cutplane.showElementColors(true);
       cutplane.showElements(true);
       cutplane.showEdges(true);
       cutplane.setGlobalElementColor(Color.BLUE);
       cutplane.setTransparency(0.7);
       cutplane.showTransparency(true);

       // Rotation around e-axis slider configuration
       translation_factor = new PuDouble("Translao:", this);     
       translation_factor.setDefBounds(-R, R, 0.1, 0.2);
       translation_factor.setDefValue(0.0);
       translation_factor.init();
       translation_factor.setEnabled(false);
       translation_factor.setEnabledConfigButton(false);

       // Flatten level slider
       flatten_level = new PuInteger("Nvel:", this);     
       flatten_level.setDefBounds(0, 100, 2, 4);
       flatten_level.setDefValue(0);
       flatten_level.init();
       flatten_level.setEnabled(false);
       flatten_level.setEnabledConfigButton(false);

       // Model parameter sliders
       m_modelingType = PwModel.PUNCH;
       m_parameterTrunc = 0.3;
       m_parameterStellate = 1.0;
       m_parameterPunch = 0.6;
       m_truncParamLowerBound = 0.01;
       m_truncParamUpperBound = 0.5;
       m_stellateParamLowerBound = -5.0;
       m_stellateParamUpperBound = 5.0;
       m_punchParamLowerBound = 0.0;
       m_punchParamUpperBound = 1.0;
       m_parameter = new PuDouble("Parmetro:", this);
       m_parameter.setDefBounds(m_punchParamLowerBound,
                                m_punchParamUpperBound, 0.01, 0.1);
       m_parameter.setDefValue(m_parameterPunch);
       m_parameter.init();      
       m_parameter.setEnabled(false);
       m_parameter.setEnabledConfigButton(false);
   }; // End of init()


   //////////////////////////////////////////////////////////////////////////
   // updateCutPlane()
   //////////////////////////////////////////////////////////////////////////
   public void updateCutPlane()
   {
      // Restore original coordinates
      cutplane.setVertex(0,  R,  R, 0);
      cutplane.setVertex(1, -R,  R, 0);
      cutplane.setVertex(2, -R, -R, 0);
      cutplane.setVertex(3,  R, -R, 0);

      // e is the unit normal of the cut plane 
      PdVector e = new PdVector(picked_point.getVertex(1).getEntry(0),
                                picked_point.getVertex(1).getEntry(1),
                                picked_point.getVertex(1).getEntry(2));
      e.normalize();

      // Using quaternions to compute the inclinitation of the cut plane
      PdVector z = new PdVector(0, 0, 1);
      double a = e.angle(e, z)*Math.PI/180.0;
      double ca = Math.cos(a/2.0);
      double sa = Math.sin(a/2.0);
      Quaternion qo = new Quaternion(ca, +sa*rotationAxis.getEntry(0),
                                         +sa*rotationAxis.getEntry(1),
                                         +sa*rotationAxis.getEntry(2));
      Quaternion qc = new Quaternion(ca, -sa*rotationAxis.getEntry(0),
                                         -sa*rotationAxis.getEntry(1),
                                         -sa*rotationAxis.getEntry(2));
      Quaternion r = new Quaternion();
      Quaternion w = new Quaternion();
      PdVector d = new PdVector();

      for (int i = 0; i < cutplane.getNumVertices(); i++)
      {
         w.setComponent(0.0, cutplane.getVertex(i).getEntry(0),
                             cutplane.getVertex(i).getEntry(1),
                             cutplane.getVertex(i).getEntry(2));
         r.setComponent(qo);
         r.multiply(w);
         r.multiply(qc);

         d.setEntry(0, e.getEntry(0));
         d.setEntry(1, e.getEntry(1));
         d.setEntry(2, e.getEntry(2));
         d.multScalar(translation_factor.getValue());
         d.add(r.getVectorPart());
         cutplane.setVertex(i, d);
      };
      cutplane.update(cutplane);
     
      // The cut operation is defining using clippiing by a level set of
      // a linear function
      double eu = Math.round(e.getEntry(0)*100.0)/100.0;
      double ev = Math.round(e.getEntry(1)*100.0)/100.0;
      double ew = Math.round(e.getEntry(2)*100.0)/100.0;
      if (eu > 0.0)
      {
         m_defClipFunc =  Double.toString(Math.abs(eu)) + "*u";
      }
      else
      {
         m_defClipFunc = "-" + Double.toString(Math.abs(eu)) + "*u";
      };
      if (ev > 0.0)
      {
         m_defClipFunc = 
                   m_defClipFunc + "+" + Double.toString(Math.abs(ev)) + "*v";
      }
      else
      {
         m_defClipFunc = 
                   m_defClipFunc + "-" + Double.toString(Math.abs(ev)) + "*v";
      };
      if (ew > 0.0)
      {
         m_defClipFunc = 
                   m_defClipFunc + "+" + Double.toString(Math.abs(ew)) + "*w";
      }
      else
      {
         m_defClipFunc = 
                   m_defClipFunc + "-" + Double.toString(Math.abs(ew)) + "*w";
      };
      m_pwClip.setFunction(m_defClipFunc);
      m_pwClip.setLevel(translation_factor.getValue()); 
      m_pwClip.clipAtFunction();
      platonic.update(platonic);
   }; // End of updateCutPlane()


   //////////////////////////////////////////////////////////////////////////
   // setModelingType()
   //////////////////////////////////////////////////////////////////////////
   public void setModelingType(int i)
   {
       if (m_modelingType == PwModel.TRUNCATEFILL)
       {
           m_truncParamUpperBound = m_parameter.getMax();
           m_truncParamLowerBound = m_parameter.getMin();
       } else
       if (m_modelingType == PwModel.TRUNCATE)
       {
           m_truncParamUpperBound = m_parameter.getMax();
           m_truncParamLowerBound = m_parameter.getMin();
       } else
       if (m_modelingType == PwModel.STELLATE)
       {
           m_stellateParamUpperBound = m_parameter.getMax();
           m_stellateParamLowerBound = m_parameter.getMin();
       } else
       if (m_modelingType == PwModel.PUNCH)
       {
           m_punchParamUpperBound = m_parameter.getMax();
           m_punchParamLowerBound = m_parameter.getMin();
       };

       if (i == PwModel.TRUNCATEFILL)
       {
           m_parameter.setBounds(m_truncParamLowerBound,
                                                      m_truncParamUpperBound);
           m_parameter.setValue(m_parameterTrunc);
       } else
       if (i == PwModel.TRUNCATE)
       {
           m_parameter.setBounds(m_truncParamLowerBound,
                                                      m_truncParamUpperBound);
           m_parameter.setValue(m_parameterTrunc);
       } else
       if (i == PwModel.STELLATE)
       {
           m_parameter.setBounds(m_stellateParamLowerBound,
                                                   m_stellateParamUpperBound);
           m_parameter.setValue(m_parameterStellate);
       } else
       if (i == PwModel.PUNCH)
       {
           m_parameter.setBounds(m_punchParamLowerBound,
                                                      m_punchParamUpperBound);
           m_parameter.setValue(m_parameterPunch);
       };

       m_modelingType = i;
   }; // End of setModelingType()


   //////////////////////////////////////////////////////////////////////////
   // Things to do when the class is started
   //////////////////////////////////////////////////////////////////////////
   public void start()
   {
      super.start();
      addGeometry(circumsphere); // We use the circumsphere to take pick
                                 // points for the cut plane operation.
                                 // So, it must be included first.

      // Necessary to apply PwWorkshop operations
      PwCleanMesh.identifyVertices(platonic, 1E-010D);
      platonic.makeNeighbour();

      addGeometry(platonic);
      selectGeometry(circumsphere);
      removeGeometry(circumsphere);

      update(this);

      disp = getDisplay(); // The PwUnfold needs a reference to the
                           // display in order to work properly.
   }; // End of start()


   /////////////////////////////////////////////////////////////////////////
   // Update the class whenever a child has changed. Method is usually
   // invoked from the children.
   /////////////////////////////////////////////////////////////////////////
   public boolean update(Object event)
   {
      if (event == null)
      {
         return(true);
      };

      // Cut plane translation factor
      if (event == translation_factor)
      {
         updateCutPlane();

         return(true);
      };

      // Unfold level
      if (event == flatten_level)
      {
         //////////////////////////////////////////////////////////////////////
         // BEGIN OF PART 6 (THE FIXED ELEMENT)
         //////////////////////////////////////////////////////////////////////
         // The cube
         if (solidName.equals(cube_trans[idiom]))
         {
            m_pwUnfold.setFixedElement(0);
         };

         // The regular thetrahedron
         if (solidName.equals(tetrahedron_trans[idiom]))
         {
            m_pwUnfold.setFixedElement(0);
         };
         //////////////////////////////////////////////////////////////////////
         // END OF PART 6
         /////////////////////////////////////////////////////////////////////
         m_pwUnfold.flatten(flatten_level.getValue()/100.0);
         platonic.update(platonic);
         return(true);
      };

      // Model operation
      if (event == m_parameter)
      {
         if ((m_modelingType == PwModel.TRUNCATE) ||
             (m_modelingType == PwModel.TRUNCATEFILL))
         {
            m_parameterTrunc = m_parameter.getValue();
            platonic.copy(m_pwModel.getSavedGeometry()); 
            m_pwModel.truncate(platonic, m_parameterTrunc, trn_fill);
         };

         if (m_modelingType == PwModel.PUNCH)
         {
            m_parameterPunch = m_parameter.getValue();
            m_pwModel.setModelingType(PwModel.PUNCH);
            platonic.copy(m_pwModel.getSavedGeometry()); 
            m_pwModel.punch(platonic, m_parameterPunch);
         };

         if (m_modelingType == PwModel.STELLATE)
         {
            m_parameterStellate = m_parameter.getValue();
            m_pwModel.setModelingType(PwModel.STELLATE);
            platonic.copy(m_pwModel.getSavedGeometry()); 
            m_pwModel.stellate(platonic, m_parameterStellate);
         };

         platonic.update(platonic);

         return(true);
      };

      return(super.update(event));
   }; // End of update()


   /////////////////////////////////////////////////////////////////////////
   // When a PwWorkshop operation is cancelled, the restore copy looses its
   // its initial properties (like  vertices  names).  So,  this  procedure
   // restores the initial configuration of the main solid
   /////////////////////////////////////////////////////////////////////////
   public void restoreInitialConfiguration()
   {
      // Restore vertices names
	  // XXX
	   
	  if (solidName.equals(cube_trans[idiom]))
	  {
		  String[] labels = {"A", "B", "C", "D", "E", "F", "G", "H"};
		  for (int i = 0; i < numberOfVertices; i++)
		  {
			  platonic.getVertex(i).setName(labels[i]);			  
		  }		  
	  }
	  
	  if (solidName.equals(tetrahedron_trans[idiom]))
	  {
		  String[] labels = {"A", "B", "C", "D"};
		  for (int i = 0; i < numberOfVertices; i++)
		  {
			  platonic.getVertex(i).setName(labels[i]);			  
		  }		  
	  }
	  
	  if (solidName.equals(octahedron_trans[idiom]))
	  {
          String[] labels = {"E", "A", "B", "C", "D", "F"};		                       
		  for (int i = 0; i < numberOfVertices; i++)
		  {
			  platonic.getVertex(i).setName(labels[i]);			  
		  }		  
	  }	  

	  if (solidName.equals(dodecahedron_trans[idiom]))
	  {
          String[] labels = {"A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "U", "V", "Q", "T", "S", "L", "M", "N", "O", "P"};		                       
		  for (int i = 0; i < numberOfVertices; i++)
		  {
			  platonic.getVertex(i).setName(labels[i]);			  
		  }		
	  }	  

	  if (solidName.equals(icosahedron_trans[idiom]))
	  {
          String[] labels = {"A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "L", "M"};		                       
		  for (int i = 0; i < numberOfVertices; i++)
		  {
			  platonic.getVertex(i).setName(labels[i]);			  
		  }		
	  }	  
	  	  
/*
      // Restore initial configuration
      platonic.setTransparency(0.5);
      platonic.showVertices(true);
      platonic.showTransparency(true);
      platonic.showElementColors(true);
      platonic.showElements(true);
      platonic.showEdges(true);
      platonic.showVertexOutline(true);
      platonic.showVertexOutline(true);
      platonic.showVertexLabels(true);
      platonic.setGlobalElementColor(Color.RED);
 */

      platonic.update(platonic);
    }; // End of restoreInitialConfiguration()


   /***************************************************************************
    ***************************************************************************
    **  Begin of PvPickListenerIf Implementation
    ***************************************************************************
    ***************************************************************************/

    //////////////////////////////////////////////////////////////////////////
    // The name of a listeners allows the display to issue verbal debug
    // messages.
    //////////////////////////////////////////////////////////////////////////
    public String getName()
    {
       return("EarthPicker");
    }; // End of getName()

    //////////////////////////////////////////////////////////////////////////
    // Get a location in the display with 2d display and 3d world coordinates.
    // Method is called when display is in mode PvDisplayIf.MODE_DISPLAY_PICK.
    //////////////////////////////////////////////////////////////////////////
    public void pickDisplay(PvPickEvent pos)
    {
       // PsDebug.message("pickDisplay() called ...");
       // System.out.println("pickDisplay() called ...");
    }; // End of pickDisplay()

    //////////////////////////////////////////////////////////////////////////
    // Drag a location in the display with 2d display and 3d world coordinates.
    // Method is called when display is in mode PvDisplayIf.MODE_DISPLAY_PICK.
    //////////////////////////////////////////////////////////////////////////
    public void dragDisplay(PvPickEvent pos)
    {
       // PsDebug.message("pickDisplay() called ...");
       // System.out.println("pickDisplay() called ...");
    }; // End of dragDisplay()

    //////////////////////////////////////////////////////////////////////////
    // Pick an arbitrary point on a geometry, point may lie inside an element.
    // Method is called when display is in mode PvDisplayIf.MODE_INITIAL_PICK
    // or if temporarily the i-key is pressed, and any pixel in the display is
    // picked.
    //////////////////////////////////////////////////////////////////////////
    public void pickInitial(PvPickEvent pos)
    {
       // PsDebug.message("pickInitial() called ...");
       // System.out.println("pickInitial() called ...");
       double x = pos.getVertex().getEntry(0);
       double y = pos.getVertex().getEntry(1);
       double z = pos.getVertex().getEntry(2);
       double norm = Math.sqrt(x*x + y*y + z*z);
       if (norm == 0.0)
       {
          double lat_in_rad = 45.0*Math.PI/180.0;
          double lon_in_rad = 90.0*Math.PI/180.0;
          x = R*Math.cos(lat_in_rad)*Math.cos(lon_in_rad);
          y = R*Math.cos(lat_in_rad)*Math.sin(lon_in_rad);
          z = R*Math.sin(lat_in_rad);
          rotationAxis.setEntry(0, x);
          rotationAxis.setEntry(1, y);
          rotationAxis.setEntry(2, z);

          PdVector v1 = new PdVector(0, 0, 1);
          PdVector v2 = new PdVector(x, y, z);
          rotationAxis.cross(v1, v2);
          rotationAxis.normalize();
       }
       else
       {
          x /= norm;
          y /= norm;
          z /= norm;
          rotationAxis.setEntry(0, x);
          rotationAxis.setEntry(1, y);
          rotationAxis.setEntry(2, z);

          PdVector v1 = new PdVector(0, 0, 1);
          PdVector v2 = new PdVector(x, y, z);
          rotationAxis.cross(v1, v2);
          rotationAxis.normalize();

          x *= R;
          y *= R;
          z *= R;
       };

       picked_point.setVertex(1, x, y, z);
       picked_point.update(picked_point);
       updateCutPlane();
    }; // End of pickInitial()

    //////////////////////////////////////////////////////////////////////////
    // Drag an arbitrary point along a geometry, point may lie inside an
    // element. Method is called when display is in mode
    // PvDisplayIf.MODE_INITIAL_PICK or if temporarily the i-key is pressed,
    // and any pixel in the display is dragged.
    //////////////////////////////////////////////////////////////////////////
    public void dragInitial(PvPickEvent pos)
    {
       // PsDebug.message("dragInitial() called ...");
       // System.out.println("dragInitial() called ...");
       double x = pos.getVertex().getEntry(0);
       double y = pos.getVertex().getEntry(1);
       double z = pos.getVertex().getEntry(2);
       double norm = Math.sqrt(x*x + y*y + z*z);
       if (norm == 0.0)
       {
          double lat_in_rad = 45.0*Math.PI/180.0;
          double lon_in_rad = 90.0*Math.PI/180.0;
          x = R*Math.cos(lat_in_rad)*Math.cos(lon_in_rad);
          y = R*Math.cos(lat_in_rad)*Math.sin(lon_in_rad);
          z = R*Math.sin(lat_in_rad);
          rotationAxis.setEntry(0, x);
          rotationAxis.setEntry(1, y);
          rotationAxis.setEntry(2, z);

          PdVector v1 = new PdVector(0, 0, 1);
          PdVector v2 = new PdVector(x, y, z);
          rotationAxis.cross(v1, v2);
          rotationAxis.normalize();
       }
       else
       {
          x /= norm;
          y /= norm;
          z /= norm;
          rotationAxis.setEntry(0, x);
          rotationAxis.setEntry(1, y);
          rotationAxis.setEntry(2, z);

          PdVector v1 = new PdVector(0, 0, 1);
          PdVector v2 = new PdVector(x, y, z);
          rotationAxis.cross(v1, v2);
          rotationAxis.normalize();

          x *= R;
          y *= R;
          z *= R;
       };

       picked_point.setVertex(1, x, y, z);
       picked_point.update(picked_point);
       updateCutPlane();
    }; // End of dragInitial()

    //////////////////////////////////////////////////////////////////////////
    // Get a picked vertex of a geometry.
    // Method is called when display is in mode PvDisplayIf.MODE_PICK
    // or if temporarily the p-key is pressed, and a vertex picked.
    //////////////////////////////////////////////////////////////////////////
    public void pickVertex(PgGeometryIf geom, int index, PdVector vertex)
    {
        // PsDebug.message("pickVertex() called ...");
        // System.out.println("pickVertex() called ...");
    }; // End of dragInitial();

    //////////////////////////////////////////////////////////////////////////
    // Method is called when display is in mode PvDisplayIf.MODE_PICK
    // or if temporarily the p-key is pressed, and a vertex dragged.
    //////////////////////////////////////////////////////////////////////////
    public void dragVertex(PgGeometryIf a_geom, int index, PdVector vertex)
    {
       // PsDebug.message("dragVertex() called ...");
       // System.out.println("dragVertex() called ...");
    }; // End of dragVertex()

    //////////////////////////////////////////////////////////////////////////
    // Mark a set of vertices of a geometry within a given bounding box.
    // Method is called when display is in mode PvDisplayIf.MODE_MARK
    // or if temporarily the m-key is pressed, and a rectangle is drawn.
    //////////////////////////////////////////////////////////////////////////
    public void markVertices(PvPickEvent markBox)
    {
       // PsDebug.message("markVertices() called ...");
       // System.out.println("markVertices() called ...");
    }; // End of markVertices()

    //////////////////////////////////////////////////////////////////////////
    // Unmark a set of vertices of a geometry within a given bounding box.
    // Method is called when display is in mode PvDisplayIf.MODE_UNMARK
    // or if temporarily the u-key is pressed, and a rectangle is drawn.
    //////////////////////////////////////////////////////////////////////////
    public void unmarkVertices(PvPickEvent markBox)
    {
       // PsDebug.message("unmarkVertices() called ...");
       // System.out.println("unmarkVertices() called ...");
    }; // End of unmarkVertices()

   /***************************************************************************
    ***************************************************************************
    **  End of PvPickListenerIf Implementation
    ***************************************************************************
    ***************************************************************************/

}; // End of PjPlatonic classs
