///////////////////////////////////////////////////////////////////////////////
// PjPlatonic Interface class
//
// Last update: 13/01/2008
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Native JAVA imports
///////////////////////////////////////////////////////////////////////////////
import java.awt.Panel;
import java.awt.TextField;
import java.awt.FlowLayout;
import java.awt.Label;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.Checkbox;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

///////////////////////////////////////////////////////////////////////////////
// JavaView imports
///////////////////////////////////////////////////////////////////////////////
import jv.geom.PgElementSet;
import jv.object.PsPanel;
import jv.objectGui.PsTabPanel;
import jv.object.PsUpdateIf;
import jv.project.PjProject_IP;
import jvx.geom.PwCleanMesh;
import jvx.geom.PwClip;
import jvx.geom.PwModel;
import jvx.geom.PwUnfold;


///////////////////////////////////////////////////////////////////////////////
// Info panel for graph tomography applet.
///////////////////////////////////////////////////////////////////////////////
public class PjPlatonic_IP extends PjProject_IP implements ActionListener, ItemListener
{
   protected PjPlatonic m_pjPlatonic;

   protected Checkbox viewElementsCB;
   protected Checkbox viewEdgesCB;
   protected Checkbox viewVerticesCB;

   protected Checkbox viewCircumsphereCB;
   protected Checkbox viewInsphereCB;

   protected Checkbox viewDualCB;
   protected Checkbox viewTetrahedronICCB;
   protected Checkbox viewIcosahedronIOCB;
   protected Checkbox viewGoldenRectangle1CB;
   protected Checkbox viewGoldenRectangle2CB;
   protected Checkbox viewGoldenRectangle3CB;
   protected Checkbox viewcubeidCB;
   protected Checkbox viewtetrahedronidCB;

   protected Checkbox enableCutPlaneCB;
   protected Checkbox viewCutPlaneCB;
   protected Checkbox posPartCB;
   protected Checkbox negPartCB;
   protected Panel m_tPanel;

   protected Checkbox enableUnfoldCB;
   protected Panel m_uPanel;

   protected Checkbox enableModelCB;
   protected Checkbox trnModelCB; // Truncate
   protected Checkbox trfModelCB; // Truncate and fill
   protected Checkbox pncModelCB; // Punch
   protected Checkbox stlModelCB; // Stellate
   protected Panel m_mPanel;

   protected PsTabPanel tabPanel;
   protected PsTabPanel cutPanel;
   protected PsTabPanel ufdPanel;
   protected PsTabPanel modPanel;

   protected int idiom = 0;
   protected String[] title_trans = {"Slidos Platnicos Regulares",
                                     "The Regular Platonic Solids"};

   protected String[] vis_vertices_trans = {"Visualizar vrtices",
                                            "Show vertices"};
   protected String[] vis_edges_trans = {"Visualizar arestas", "Show edges"};
   protected String[] vis_faces_trans = {"Visualizar faces", "Show faces"};

   protected String[] vis_circumsphere_trans =
                      {"Visualizar esfera circunscrita", "Show circumsphere"};
   protected String[] vis_insphere_trans = {"Visualizar esfera inscrita",
                                            "Show insphere"};

   protected String[] vis_dual_trans = {"Visualizar dual", "Show dual"};
   protected String[] vis_tetrahedron_ic_trans =
                                     {"Visualizar tetraedro regular inscrito",
                                      "Show inscribed regular tetrahedron"};
   protected String[] vis_icosahedron_io_trans =
                                     {"Visualizar icosaedro regular inscrito",
                                      "Show inscribed regular icosahedron"};
   protected String[] vis_golden_rectangle_1_trans =
                                     {"Visualizar retngulo ureo 1",
                                      "Show golden rectangle 1"};
   protected String[] vis_golden_rectangle_2_trans =
                                     {"Visualizar retngulo ureo 2",
                                      "Show golden rectangle 2"};
   protected String[] vis_golden_rectangle_3_trans =
                                     {"Visualizar retngulo ureo 3",
                                      "Show golden rectangle 3"};
   protected String[] vis_cube_id_trans =
                                     {"Visualizar cubo inscrito",
                                      "Show inscribed cube"};
   protected String[] vis_tetrahedron_id_trans =
                                     {"Visualizar tetraedro inscrito",
                                      "Show inscribed tetrahedron"};
                                                                          
   protected String[] vis_panel_trans = {"Visualizar", "Visualize"};

   protected String[] cut_enable_trans = {"Habilitar operao de corte",
                                          "Enable cut operation"};
   protected String[] cut_view_trans = {"Visualizar plano de corte",
                                        "Show cut plane"};
   protected String[] cut_part1_trans = {"Exibir parte 1", "Show part 1"};
   protected String[] cut_part2_trans = {"Exibir parte 2", "Show part 2"};
   protected String[] cut_translation_trans = {"Translao", "Translation"};
   protected String[] cut_panel_trans = {"Cortar", "Cut"};

   protected String[] ufd_enable_trans = {"Habilitar montagem",
                                          "Enable unfold"};
   protected String[] flatten_level_trans = {"Parmetro:", "Parameter:"};
   protected String[] ufd_panel_trans = {"Planificar", "Unfold"};

   protected String[] mod_enable_trans = {"Habilitar modelagem",
                                          "Enable modelling"};
   protected String[] mod_trn_trans = {"Truncar", "Truncate"};
   protected String[] mod_trf_trans = {"Truncar e preencher",
                                       "Truncate and fill"};
   protected String[] mod_pnc_trans = {"Esburacar", "Punch"}; 
   protected String[] mod_stl_trans = {"Estrelar", "Stellate"};
   protected String[] parameter_trans = {"Parmetro:", "Parameter:"};
   protected String[] mod_panel_trans = {"Modelar", "Model"};


   ///////////////////////////////////////////////////////////////////////////
   // Default constructor
   ///////////////////////////////////////////////////////////////////////////
   public PjPlatonic_IP()
   {
      super();

      if (getClass() == PjPlatonic_IP.class)
      {
         init();
      };
   }; // End of PjPlatonic_IP()


   //////////////////////////////////////////////////////////////////////////
   // Setup the main control panel
   //////////////////////////////////////////////////////////////////////////
   public void init()
   {
      super.init();

      // Title
      addTitle(title_trans[idiom]);

      add(new Label(" "));

      tabPanel = new PsTabPanel();
      add(tabPanel);
   }; // End of init()


   //////////////////////////////////////////////////////////////////////////
   // Setup parent and add the level slider
   //////////////////////////////////////////////////////////////////////////
   public void setParent(PsUpdateIf parent)
   {
      super.setParent(parent);
      m_pjPlatonic = (PjPlatonic) parent;
      idiom = m_pjPlatonic.idiom;

      setTitle(m_pjPlatonic.solidName);

      // Visualization panel
      PsPanel visPanel = new PsPanel();      

      viewVerticesCB = new Checkbox(vis_vertices_trans[idiom]);
      viewVerticesCB.addItemListener(this);
      viewVerticesCB.setState(true);
      visPanel.add(viewVerticesCB);

      viewEdgesCB = new Checkbox(vis_edges_trans[idiom]);
      viewEdgesCB.addItemListener(this);
      viewEdgesCB.setState(true);
      visPanel.add(viewEdgesCB);

      viewElementsCB = new Checkbox(vis_faces_trans[idiom]);
      viewElementsCB.addItemListener(this);
      viewElementsCB.setState(true);
      visPanel.add(viewElementsCB);

      visPanel.addLine(1);

      viewCircumsphereCB = new Checkbox(vis_circumsphere_trans[idiom]);
      viewCircumsphereCB.addItemListener(this);
      viewCircumsphereCB.setState(false);
      visPanel.add(viewCircumsphereCB);

      viewInsphereCB = new Checkbox(vis_insphere_trans[idiom]);
      viewInsphereCB.addItemListener(this);
      viewInsphereCB.setState(false);
      visPanel.add(viewInsphereCB);

      visPanel.addLine(1);

      viewDualCB = new Checkbox(vis_dual_trans[idiom]);
      viewDualCB.addItemListener(this);
      viewDualCB.setState(false);
      visPanel.add(viewDualCB);

      if (m_pjPlatonic.solidName.equals(m_pjPlatonic.cube_trans[idiom]))
      {
         viewTetrahedronICCB = new Checkbox(vis_tetrahedron_ic_trans[idiom]);
         viewTetrahedronICCB.addItemListener(this);
         viewTetrahedronICCB.setState(false);
         visPanel.add(viewTetrahedronICCB);
      };

      if (m_pjPlatonic.solidName.equals(m_pjPlatonic.octahedron_trans[idiom]))
      {
         viewIcosahedronIOCB = new Checkbox(vis_icosahedron_io_trans[idiom]);
         viewIcosahedronIOCB.addItemListener(this);
         viewIcosahedronIOCB.setState(false);
         visPanel.add(viewIcosahedronIOCB);
      };

      if (m_pjPlatonic.solidName.equals(m_pjPlatonic.dodecahedron_trans[idiom]))
      {
         viewtetrahedronidCB = new Checkbox(vis_tetrahedron_id_trans[idiom]);
         viewtetrahedronidCB.addItemListener(this);
         viewtetrahedronidCB.setState(false);
         visPanel.add(viewtetrahedronidCB);
                 
         viewcubeidCB = new Checkbox(vis_cube_id_trans[idiom]);
         viewcubeidCB.addItemListener(this);
         viewcubeidCB.setState(false);
         visPanel.add(viewcubeidCB);

         visPanel.addLine(1);          
          
         viewGoldenRectangle1CB =
                            new Checkbox(vis_golden_rectangle_1_trans[idiom]);
         viewGoldenRectangle1CB.addItemListener(this);
         viewGoldenRectangle1CB.setState(false);
         visPanel.add(viewGoldenRectangle1CB);
          
         viewGoldenRectangle2CB =
                            new Checkbox(vis_golden_rectangle_2_trans[idiom]);
         viewGoldenRectangle2CB.addItemListener(this);
         viewGoldenRectangle2CB.setState(false);
         visPanel.add(viewGoldenRectangle2CB);
          
         viewGoldenRectangle3CB =
                            new Checkbox(vis_golden_rectangle_3_trans[idiom]);
         viewGoldenRectangle3CB.addItemListener(this);
         viewGoldenRectangle3CB.setState(false);
         visPanel.add(viewGoldenRectangle3CB);
      };


      if (m_pjPlatonic.solidName.equals(m_pjPlatonic.icosahedron_trans[idiom]))
      {
         visPanel.addLine(1); 

         viewGoldenRectangle1CB =
                            new Checkbox(vis_golden_rectangle_1_trans[idiom]);
         viewGoldenRectangle1CB.addItemListener(this);
         viewGoldenRectangle1CB.setState(false);
         visPanel.add(viewGoldenRectangle1CB);
          
         viewGoldenRectangle2CB =
                            new Checkbox(vis_golden_rectangle_2_trans[idiom]);
         viewGoldenRectangle2CB.addItemListener(this);
         viewGoldenRectangle2CB.setState(false);
         visPanel.add(viewGoldenRectangle2CB);
          
         viewGoldenRectangle3CB =
                            new Checkbox(vis_golden_rectangle_3_trans[idiom]);
         viewGoldenRectangle3CB.addItemListener(this);
         viewGoldenRectangle3CB.setState(false);
         visPanel.add(viewGoldenRectangle3CB);
      };

      visPanel.addLine(1);

      tabPanel.addPanel(vis_panel_trans[idiom], visPanel);

      // Cut plane panel
      PsPanel cutPanel = new PsPanel();      
      
      enableCutPlaneCB = new Checkbox(cut_enable_trans[idiom]);
      enableCutPlaneCB.addItemListener(this);
      enableCutPlaneCB.setState(false);
      cutPanel.add(enableCutPlaneCB);

      viewCutPlaneCB = new Checkbox(cut_view_trans[idiom]);
      viewCutPlaneCB.addItemListener(this);
      viewCutPlaneCB.setState(true);
      viewCutPlaneCB.setEnabled(false);
      cutPanel.add(viewCutPlaneCB);

      negPartCB = new Checkbox(cut_part1_trans[idiom]);
      negPartCB.addItemListener(this);
      negPartCB.setState(true);
      negPartCB.setEnabled(false);

      posPartCB = new Checkbox(cut_part2_trans[idiom]);
      posPartCB.addItemListener(this);
      posPartCB.setState(false);
      posPartCB.setEnabled(false);

      PsPanel partPanel = new PsPanel();
      partPanel.setLayout(new GridLayout(1, 2));
      partPanel.add(negPartCB);
      partPanel.add(posPartCB);

      cutPanel.add(partPanel);

      m_tPanel = new Panel();
      m_tPanel.setLayout(new GridLayout(1, 1));
      cutPanel.add(m_tPanel);

      cutPanel.addLine(1);

      tabPanel.addPanel(cut_panel_trans[idiom], cutPanel);

      m_tPanel.removeAll();
      m_pjPlatonic.translation_factor.setName(cut_translation_trans[idiom]);
      m_tPanel.add(m_pjPlatonic.translation_factor.newInspector(PsPanel.INFO_EXT));

      // Unfold panel
      PsPanel ufdPanel = new PsPanel();

      enableUnfoldCB = new Checkbox(ufd_enable_trans[idiom]);
      enableUnfoldCB.addItemListener(this);
      enableUnfoldCB.setState(false);

      ufdPanel.add(enableUnfoldCB);

      m_uPanel = new Panel();
      m_uPanel.setLayout(new GridLayout(1, 1));
      ufdPanel.add(m_uPanel);

      tabPanel.addPanel(ufd_panel_trans[idiom], ufdPanel);

      m_uPanel.removeAll();
      m_pjPlatonic.translation_factor.setName(flatten_level_trans[idiom]);
      m_uPanel.add(m_pjPlatonic.flatten_level.newInspector(PsPanel.INFO_EXT));

      ufdPanel.addLine(1);

      // Model panel 
      PsPanel modPanel = new PsPanel();

      enableModelCB = new Checkbox(mod_enable_trans[idiom]);
      enableModelCB.addItemListener(this);
      enableModelCB.setState(false);
      modPanel.add(enableModelCB);

      trnModelCB = new Checkbox(mod_trn_trans[idiom]);
      trnModelCB.addItemListener(this);
      trnModelCB.setState(false);
      trnModelCB.setEnabled(false);
      modPanel.add(trnModelCB);

      trfModelCB = new Checkbox(mod_trf_trans[idiom]);
      trfModelCB.addItemListener(this);
      trfModelCB.setState(false);
      trfModelCB.setEnabled(false);
      modPanel.add(trfModelCB);

      pncModelCB = new Checkbox(mod_pnc_trans[idiom]);
      pncModelCB.addItemListener(this);
      pncModelCB.setState(true);
      pncModelCB.setEnabled(false);
      modPanel.add(pncModelCB);

      stlModelCB = new Checkbox(mod_stl_trans[idiom]);
      stlModelCB.addItemListener(this);
      stlModelCB.setState(false);
      stlModelCB.setEnabled(false);
      modPanel.add(stlModelCB);

      m_mPanel = new Panel();
      m_mPanel.setLayout(new GridLayout(1, 1));
      modPanel.add(m_mPanel);
      
      m_pjPlatonic.m_parameter.setName(parameter_trans[idiom]);

      m_mPanel.removeAll();
      m_mPanel.add(m_pjPlatonic.m_parameter.newInspector(PsPanel.INFO_EXT));

      modPanel.addLine(1);

      tabPanel.addPanel(mod_panel_trans[idiom], modPanel);

      // Validate 
      tabPanel.validate();
       
      super.setParent(parent);
   }; // End of setParent()


   //////////////////////////////////////////////////////////////////////////
   // Update the panel whenever the parent has changed somewhere else
   //////////////////////////////////////////////////////////////////////////
   public boolean update(Object  event)
   {
      if (m_pjPlatonic == event)
      {
         // Fill here!

         return(true);
      };

      return(super.update(event));
   }; // End of update()


   //////////////////////////////////////////////////////////////////////////
   // Handle action events invoked from text fields
   //////////////////////////////////////////////////////////////////////////
   public void actionPerformed(ActionEvent event)
   {
      if (m_pjPlatonic == null)
      {
          return;
      };

      Object source = event.getSource();

      // Fill here!

   }; // End of actionPerformed()


   //////////////////////////////////////////////////////////////////////////
   // Handle action events invoked from itens
   //////////////////////////////////////////////////////////////////////////
   public void itemStateChanged(ItemEvent event)
   {
       if (m_pjPlatonic == null)
       {
          return;
       };

       //////////////////////////////////////////////////////////////////////
       // Visualization panel
       //////////////////////////////////////////////////////////////////////
       Object source = event.getSource();
       if (source == viewElementsCB)
       {
          if (viewElementsCB.getState() == true)
          {
             m_pjPlatonic.platonic.showElements(true);
             m_pjPlatonic.platonic.update(m_pjPlatonic.platonic);

             // Synchronize visualization properties of the original and
             // PwWorkshop objects.
             if (enableCutPlaneCB.getState() == true) 
             {
                PgElementSet original =
                      (PgElementSet) m_pjPlatonic.m_pwClip.getSavedGeometry();
                original.showElements(true);
             };

             if (enableUnfoldCB.getState() == true) 
             {
                PgElementSet original =
                    (PgElementSet) m_pjPlatonic.m_pwUnfold.getSavedGeometry();
                original.showElements(true);
             };

             if (enableModelCB.getState() == true) 
             {
                PgElementSet original =
                     (PgElementSet) m_pjPlatonic.m_pwModel.getSavedGeometry();
                original.showElements(true);
             };
          }
          else
          {
             m_pjPlatonic.platonic.showElements(false);
             m_pjPlatonic.platonic.update(m_pjPlatonic.platonic);

             // Synchronize visualization properties of the original and
             // PwWorkshop objects.
             if (enableCutPlaneCB.getState() == true) 
             {
                PgElementSet original =
                      (PgElementSet) m_pjPlatonic.m_pwClip.getSavedGeometry();
                original.showElements(false);
             };

             if (enableUnfoldCB.getState() == true) 
             {
                PgElementSet original =
                    (PgElementSet) m_pjPlatonic.m_pwUnfold.getSavedGeometry();
                original.showElements(false);
             };

             if (enableModelCB.getState() == true) 
             {
                PgElementSet original =
                     (PgElementSet) m_pjPlatonic.m_pwModel.getSavedGeometry();
                original.showElements(false);
             };
          };
          return;
       }; // End of if (source == viewElementsCB)

       if (source == viewEdgesCB)
       {
          if (viewEdgesCB.getState() == true)
          {
             m_pjPlatonic.platonic.showEdges(true);
             m_pjPlatonic.platonic.update(m_pjPlatonic.platonic);

             // Synchronize visualization properties of the original and
             // PwWorkshop objects.
             if (enableCutPlaneCB.getState() == true) 
             {
                PgElementSet original =
                      (PgElementSet) m_pjPlatonic.m_pwClip.getSavedGeometry();
                original.showEdges(true);
             };

             if (enableUnfoldCB.getState() == true) 
             {
                PgElementSet original =
                    (PgElementSet) m_pjPlatonic.m_pwUnfold.getSavedGeometry();
                original.showEdges(true);
             };

             if (enableModelCB.getState() == true) 
             {
                PgElementSet original =
                     (PgElementSet) m_pjPlatonic.m_pwModel.getSavedGeometry();
                original.showEdges(true);
             };
          }
          else
          {
             m_pjPlatonic.platonic.showEdges(false);
             m_pjPlatonic.platonic.update(m_pjPlatonic.platonic);

             // Synchronize visualization properties of the original and
             // PwWorkshop objects.
             if (enableCutPlaneCB.getState() == true) 
             {
                PgElementSet original =
                      (PgElementSet) m_pjPlatonic.m_pwClip.getSavedGeometry();
                original.showEdges(false);
             };

             if (enableUnfoldCB.getState() == true) 
             {
                PgElementSet original =
                    (PgElementSet) m_pjPlatonic.m_pwUnfold.getSavedGeometry();
                original.showEdges(false);
             };

             if (enableModelCB.getState() == true) 
             {
                PgElementSet original =
                     (PgElementSet) m_pjPlatonic.m_pwModel.getSavedGeometry();
                original.showEdges(false);
             };
          };
          return;
       }; // End of if (source == viewEdgesCB)

       if (source == viewVerticesCB)
       {
          if (viewVerticesCB.getState() == true)
          {
             m_pjPlatonic.platonic.showVertices(true);
             m_pjPlatonic.platonic.update(m_pjPlatonic.platonic);

             // Synchronize visualization properties of the original and
             // PwWorkshop objects.
             if (enableCutPlaneCB.getState() == true) 
             {
                PgElementSet original =
                      (PgElementSet) m_pjPlatonic.m_pwClip.getSavedGeometry();
                original.showVertices(true);
             };

             if (enableUnfoldCB.getState() == true) 
             {
                PgElementSet original =
                    (PgElementSet) m_pjPlatonic.m_pwUnfold.getSavedGeometry();
                original.showVertices(true);
             };

             if (enableModelCB.getState() == true) 
             {
                PgElementSet original =
                     (PgElementSet) m_pjPlatonic.m_pwModel.getSavedGeometry();
                original.showVertices(true);
             };
          }
          else
          {
             m_pjPlatonic.platonic.showVertices(false);
             m_pjPlatonic.platonic.update(m_pjPlatonic.platonic);

             // Synchronize visualization properties of the original and
             // PwWorkshop objects.
             if (enableCutPlaneCB.getState() == true) 
             {
                PgElementSet original =
                      (PgElementSet) m_pjPlatonic.m_pwClip.getSavedGeometry();
                original.showVertices(false);
             };

             if (enableUnfoldCB.getState() == true) 
             {
                PgElementSet original =
                    (PgElementSet) m_pjPlatonic.m_pwUnfold.getSavedGeometry();
                original.showVertices(false);
             };

             if (enableModelCB.getState() == true) 
             {
                PgElementSet original =
                     (PgElementSet) m_pjPlatonic.m_pwModel.getSavedGeometry();
                original.showVertices(false);
             };
          };
          return;
       }; // End of if (source == viewVerticesCB)

       if (source == viewCircumsphereCB)
       {
          if (viewCircumsphereCB.getState() == true)
          {
             if (enableCutPlaneCB.getState() == true)
             {
                m_pjPlatonic.circumsphere.setTransparency(0.9);
                m_pjPlatonic.circumsphere.update(m_pjPlatonic.circumsphere);
                viewCutPlaneCB.setEnabled(true);
                posPartCB.setEnabled(true);
                negPartCB.setEnabled(true);
             }
             else
             {
                m_pjPlatonic.addGeometry(m_pjPlatonic.circumsphere);
                m_pjPlatonic.selectGeometry(m_pjPlatonic.circumsphere);
                viewCutPlaneCB.setEnabled(false);
                posPartCB.setEnabled(false);
                negPartCB.setEnabled(false);
             };
          }
          else
          {
             if (enableCutPlaneCB.getState() == true)
             {
                m_pjPlatonic.circumsphere.setTransparency(1.0);
                m_pjPlatonic.circumsphere.update(m_pjPlatonic.circumsphere);
                viewCutPlaneCB.setEnabled(true);
                posPartCB.setEnabled(true);
                negPartCB.setEnabled(true);
             }
             else
             {
                m_pjPlatonic.removeGeometry(m_pjPlatonic.circumsphere);
                viewCutPlaneCB.setEnabled(false);
                posPartCB.setEnabled(false);
                negPartCB.setEnabled(false);
             };
          };
          return;
       }; // End of if (source == viewCircumsphereCB)

       if (source == viewInsphereCB)
       {
          if (viewInsphereCB.getState() == true)
          {
             m_pjPlatonic.addGeometry(m_pjPlatonic.insphere);
             m_pjPlatonic.addGeometry(m_pjPlatonic.dualvertices);
          }
          else
          {
             m_pjPlatonic.removeGeometry(m_pjPlatonic.insphere);
             m_pjPlatonic.removeGeometry(m_pjPlatonic.dualvertices);
          };
          return;
       }; // End of if (source == viewInsphereCB) 

       if (source == viewDualCB)
       {
          if (viewDualCB.getState() == true)
          {
             m_pjPlatonic.addGeometry(m_pjPlatonic.dual);
          }
          else
          {
             m_pjPlatonic.removeGeometry(m_pjPlatonic.dual);
          };
          return;
       }; // End of if (source == viewDualCB)


       if ((source == viewTetrahedronICCB) &&
           (m_pjPlatonic.solidName.equals(m_pjPlatonic.cube_trans[idiom])))
       {
          if (viewTetrahedronICCB.getState() == true)
          {
             m_pjPlatonic.addGeometry(m_pjPlatonic.tetrahedron_ic);
          }
          else
          {
             m_pjPlatonic.removeGeometry(m_pjPlatonic.tetrahedron_ic);
          };
          return;
       }; 
       
       // End of if ((source == viewTetrahedronICCB) &&
       //            (m_pjPlatonic.solidName.equals(m_pjPlatonic.cube_trans[idiom])))

       if ((source == viewIcosahedronIOCB) &&
           (m_pjPlatonic.solidName.equals(m_pjPlatonic.octahedron_trans[idiom])))
       {
          if (viewIcosahedronIOCB.getState() == true)
          {
             m_pjPlatonic.addGeometry(m_pjPlatonic.icosahedron_io);
          }
          else
          {
             m_pjPlatonic.removeGeometry(m_pjPlatonic.icosahedron_io);
          };
          return;
       };
       
       // End of if ((source == viewIcosahedronIOCB) &&
       //            (m_pjPlatonic.solidName.equals(m_pjPlatonic.octahedron_trans[idiom])))
          
       if ((source == viewGoldenRectangle1CB) &&
           (m_pjPlatonic.solidName.equals(m_pjPlatonic.dodecahedron_trans[idiom])))
       {
          if (viewGoldenRectangle1CB.getState() == true)
          {
             m_pjPlatonic.addGeometry(m_pjPlatonic.goldenrectangle1);
          }
          else
          {
             m_pjPlatonic.removeGeometry(m_pjPlatonic.goldenrectangle1);
          };
          return;
       }; // End of if ((source == viewGoldenRectangle1CB) &&
          //            (m_pjPlatonic.solidName.equals(m_pjPlatonic.dodecahedron_trans[idiom]))) 

       if ((source == viewGoldenRectangle2CB) &&
           (m_pjPlatonic.solidName.equals(m_pjPlatonic.dodecahedron_trans[idiom])))
       {
          if (viewGoldenRectangle2CB.getState() == true)
          {
             m_pjPlatonic.addGeometry(m_pjPlatonic.goldenrectangle2);
          }
          else
          {
             m_pjPlatonic.removeGeometry(m_pjPlatonic.goldenrectangle2);
          };
          return;
       }; // End of if ((source == viewGoldenRectangle2CB) &&
          //            (m_pjPlatonic.solidName.equals(m_pjPlatonic.dodecahedron_trans[idiom]))) 

       if ((source == viewGoldenRectangle3CB) &&
           (m_pjPlatonic.solidName.equals(m_pjPlatonic.dodecahedron_trans[idiom])))
       {
          if (viewGoldenRectangle3CB.getState() == true)
          {
             m_pjPlatonic.addGeometry(m_pjPlatonic.goldenrectangle3);
          }
          else
          {
             m_pjPlatonic.removeGeometry(m_pjPlatonic.goldenrectangle3);
          };
          return;
       }; // End of if ((source == viewGoldenRectangle3CB) &&
          //            (m_pjPlatonic.solidName.equals(m_pjPlatonic.dodecahedron_trans[idiom])))
       
       if ((source == viewGoldenRectangle1CB) &&
           (m_pjPlatonic.solidName.equals(m_pjPlatonic.icosahedron_trans[idiom])))
       {
          if (viewGoldenRectangle1CB.getState() == true)
          {
             m_pjPlatonic.addGeometry(m_pjPlatonic.goldenrectangle1);
          }
          else
          {
             m_pjPlatonic.removeGeometry(m_pjPlatonic.goldenrectangle1);
          };
          return;
       };
           
           // End of if ((source == viewtetraedronICCB) &&
       //            (m_pjPlatonic.solidName.equals(m_pjPlatonic.tetrahedron_trans[idiom])))

       if ((source == viewtetrahedronidCB) &&
           (m_pjPlatonic.solidName.equals(m_pjPlatonic.dodecahedron_trans[idiom])))
       {
          if (viewtetrahedronidCB.getState() == true)
          {
             m_pjPlatonic.addGeometry(m_pjPlatonic.tetrahedron_id);
          }
          else
          {
             m_pjPlatonic.removeGeometry(m_pjPlatonic.tetrahedron_id);
          };
          return;
       };
      
       // End of if ((source == viewcubeICCB) &&
       //            (m_pjPlatonic.solidName.equals(m_pjPlatonic.dodecahedron_trans[idiom])))

       if ((source == viewcubeidCB) &&
           (m_pjPlatonic.solidName.equals(m_pjPlatonic.dodecahedron_trans[idiom])))
       {
          if (viewcubeidCB.getState() == true)
          {
             m_pjPlatonic.addGeometry(m_pjPlatonic.cube_id);
          }
          else
          {
             m_pjPlatonic.removeGeometry(m_pjPlatonic.cube_id);
          };
          return;
       };

       // End of if ((source == viewGoldenRectangle1CB) &&
          //            (m_pjPlatonic.solidName.equals(m_pjPlatonic.icosahedron_trans[idiom]))) 

       if ((source == viewGoldenRectangle2CB) &&
           (m_pjPlatonic.solidName.equals(m_pjPlatonic.icosahedron_trans[idiom])))
       {
          if (viewGoldenRectangle2CB.getState() == true)
          {
             m_pjPlatonic.addGeometry(m_pjPlatonic.goldenrectangle2);
          }
          else
          {
             m_pjPlatonic.removeGeometry(m_pjPlatonic.goldenrectangle2);
          };
          return;
       }; // End of if ((source == viewGoldenRectangle2CB) &&
          //            (m_pjPlatonic.solidName.equals(m_pjPlatonic.icosahedron_trans[idiom]))) 

       if ((source == viewGoldenRectangle3CB) &&
           (m_pjPlatonic.solidName.equals(m_pjPlatonic.icosahedron_trans[idiom])))
       {
          if (viewGoldenRectangle3CB.getState() == true)
          {
             m_pjPlatonic.addGeometry(m_pjPlatonic.goldenrectangle3);
          }
          else
          {
             m_pjPlatonic.removeGeometry(m_pjPlatonic.goldenrectangle3);
          };
          return;
       }; // End of if ((source == viewGoldenRectangle3CB) &&
          //            (m_pjPlatonic.solidName.equals(m_pjPlatonic.icosahedron_trans[idiom])))
         

       //////////////////////////////////////////////////////////////////////
       // Cut plane panel
       //////////////////////////////////////////////////////////////////////
       if (source == enableCutPlaneCB)
       {
          if (enableCutPlaneCB.getState() == true)
          {
             // Disable the others operations, if any
             if (enableUnfoldCB.getState() == true)
             {
                 enableUnfoldCB.setState(false);
                 m_pjPlatonic.flatten_level.setEnabled(false); 
                 m_pjPlatonic.m_pwUnfold.cancel();
                 m_pjPlatonic.restoreInitialConfiguration();
                 m_pjPlatonic.platonic.showVertexLabels(true);
             };
             if (enableModelCB.getState() == true)
             {
                enableModelCB.setState(false);
                trnModelCB.setEnabled(false);
                trfModelCB.setEnabled(false);
                pncModelCB.setEnabled(false);
                stlModelCB.setEnabled(false);
                m_pjPlatonic.m_parameter.setEnabled(false); 
                m_pjPlatonic.m_pwModel.cancel();
                m_pjPlatonic.platonic.showVertexLabels(true);
                m_pjPlatonic.restoreInitialConfiguration();
             };

             // Cut plane setup
             viewCutPlaneCB.setEnabled(true);
             posPartCB.setEnabled(true);
             negPartCB.setEnabled(true);
             m_pjPlatonic.platonic.showVertexLabels(false);
             if (viewCutPlaneCB.getState() == true)
             {
                m_pjPlatonic.addGeometry(m_pjPlatonic.cutplane);
             };
             m_pjPlatonic.m_defClipFunc = "u+v+w";
             m_pjPlatonic.m_pwClip = new PwClip();
             m_pjPlatonic.m_pwClip.setFunction(m_pjPlatonic.m_defClipFunc);
             m_pjPlatonic.m_pwClip.setLevel(0.0);
             m_pjPlatonic.m_pwClip.setGeometry(m_pjPlatonic.platonic);
             if (posPartCB.getState() == true)
             {
                m_pjPlatonic.m_pwClip.setEnabledPositive(true);
             }
             else
             {
                m_pjPlatonic.m_pwClip.setEnabledPositive(false);
             };

             m_pjPlatonic.addGeometry(m_pjPlatonic.picked_point);
             m_pjPlatonic.translation_factor.setEnabled(true);
             if (viewCircumsphereCB.getState() == false)
             {
                m_pjPlatonic.addGeometry(m_pjPlatonic.circumsphere);
                m_pjPlatonic.selectGeometry(m_pjPlatonic.circumsphere);
                m_pjPlatonic.circumsphere.setTransparency(1.0);
                m_pjPlatonic.circumsphere.update(m_pjPlatonic.circumsphere);
             }
             m_pjPlatonic.updateCutPlane();
          }
          else
          {
             viewCutPlaneCB.setEnabled(false);
             posPartCB.setEnabled(false);
             negPartCB.setEnabled(false);
             m_pjPlatonic.removeGeometry(m_pjPlatonic.cutplane);

             m_pjPlatonic.m_pwClip.cancel();
             m_pjPlatonic.platonic.showVertexLabels(true);
             m_pjPlatonic.restoreInitialConfiguration();
             m_pjPlatonic.removeGeometry(m_pjPlatonic.picked_point);
             if (viewCircumsphereCB.getState() == false)
             {
                m_pjPlatonic.removeGeometry(m_pjPlatonic.circumsphere);
                m_pjPlatonic.circumsphere.setTransparency(0.9);
                m_pjPlatonic.circumsphere.update(m_pjPlatonic.circumsphere);
                m_pjPlatonic.translation_factor.setEnabled(false);
             };
          };
          return;
       }; // End of if (source == enableCutPlaneCB)

       if (source == viewCutPlaneCB)
       {
          if (viewCutPlaneCB.getState() == true)
          {
             m_pjPlatonic.addGeometry(m_pjPlatonic.cutplane);
          }
          else
          {
             m_pjPlatonic.removeGeometry(m_pjPlatonic.cutplane);
          };
          return;
       }; // End of if (source == viewCutPlaneCB)

       if (source == posPartCB)
       {
          if (posPartCB.getState() == true)
          {
             negPartCB.setState(false);
             m_pjPlatonic.m_pwClip.setEnabledPositive(true);
             m_pjPlatonic.updateCutPlane();
          }
          else
          {
             posPartCB.setState(true);
          };

          return;
       }; // End of if (source == posPartCB)

       if (source == negPartCB)
       {
          if (negPartCB.getState() == true)
          {
             posPartCB.setState(false);
             m_pjPlatonic.m_pwClip.setEnabledPositive(false);
             m_pjPlatonic.updateCutPlane();
          }
          else
          {
             negPartCB.setState(true);
          };

          return;
       }; // End of if (source == negPartCB)

       //////////////////////////////////////////////////////////////////////
       // Unfold panel 
       //////////////////////////////////////////////////////////////////////
       if (source == enableUnfoldCB)
       {
          if (enableUnfoldCB.getState() == true)
          {
             // Disable the others operations, if any
             if (enableCutPlaneCB.getState() == true)
             {
                enableCutPlaneCB.setState(false);
                viewCutPlaneCB.setEnabled(false);
                posPartCB.setEnabled(false);
                negPartCB.setEnabled(false);
                m_pjPlatonic.removeGeometry(m_pjPlatonic.cutplane);

                m_pjPlatonic.m_pwClip.cancel();
                m_pjPlatonic.platonic.showVertexLabels(true);
                m_pjPlatonic.restoreInitialConfiguration();
                m_pjPlatonic.removeGeometry(m_pjPlatonic.picked_point);
                
                if (viewCircumsphereCB.getState() == false)
                {
                   m_pjPlatonic.removeGeometry(m_pjPlatonic.circumsphere);
                   m_pjPlatonic.circumsphere.setTransparency(0.9);
                   m_pjPlatonic.circumsphere.update(m_pjPlatonic.circumsphere);
                };
                m_pjPlatonic.translation_factor.setEnabled(false);
             };
             if (enableModelCB.getState() == true)
             {
                enableModelCB.setState(false);
                trnModelCB.setEnabled(false);
                trfModelCB.setEnabled(false);
                pncModelCB.setEnabled(false);
                stlModelCB.setEnabled(false);
                m_pjPlatonic.m_parameter.setEnabled(false); 
                m_pjPlatonic.platonic.showVertexLabels(true);
                m_pjPlatonic.m_pwModel.cancel();
                m_pjPlatonic.restoreInitialConfiguration();
             };

             // Unfold setup
             m_pjPlatonic.flatten_level.setEnabled(true); 
             m_pjPlatonic.m_pwUnfold = new PwUnfold();
             m_pjPlatonic.m_pwUnfold.setGeometry(m_pjPlatonic.platonic);
             m_pjPlatonic.platonic.showVertexLabels(false);
             m_pjPlatonic.m_pwUnfold.setEnabledFixingElement(true);
             m_pjPlatonic.m_pwUnfold.setDisplay(m_pjPlatonic.disp);
                                      // The PwUnfold needs a reference to the
                                      // display in order to work properly.
             m_pjPlatonic.m_pwUnfold.flatten(m_pjPlatonic.flatten_level.getValue()/100.0);
             m_pjPlatonic.platonic.update(m_pjPlatonic.platonic);
	     // m_pjPlatonic.disp.fit();
          }
          else
          {
              m_pjPlatonic.flatten_level.setEnabled(false); 
              m_pjPlatonic.m_pwUnfold.cancel();
              m_pjPlatonic.restoreInitialConfiguration();
              m_pjPlatonic.platonic.showVertexLabels(true);
          };
          return;
       }; // End of if (source == enableUnfoldCB)

       //////////////////////////////////////////////////////////////////////
       // Model panel
       //////////////////////////////////////////////////////////////////////
       if (source == enableModelCB)
       {
          if (enableModelCB.getState() == true)
          {
             // Disable the others operations, if any
             if (enableUnfoldCB.getState() == true)
             {
                 enableUnfoldCB.setState(false);
                 m_pjPlatonic.flatten_level.setEnabled(false); 
                 m_pjPlatonic.m_pwUnfold.cancel();
                 m_pjPlatonic.restoreInitialConfiguration();
                 m_pjPlatonic.platonic.showVertexLabels(true);
             };
             if (enableCutPlaneCB.getState() == true)
             {
                enableCutPlaneCB.setState(false);
                viewCutPlaneCB.setEnabled(false);
                posPartCB.setEnabled(false);
                negPartCB.setEnabled(false);
                m_pjPlatonic.removeGeometry(m_pjPlatonic.cutplane);

                m_pjPlatonic.m_pwClip.cancel();
                m_pjPlatonic.platonic.showVertexLabels(true);
                m_pjPlatonic.restoreInitialConfiguration();
                m_pjPlatonic.removeGeometry(m_pjPlatonic.picked_point);
                
                if (viewCircumsphereCB.getState() == false)
                {
                   m_pjPlatonic.removeGeometry(m_pjPlatonic.circumsphere);
                   m_pjPlatonic.circumsphere.setTransparency(0.9);
                   m_pjPlatonic.circumsphere.update(m_pjPlatonic.circumsphere);
                };
                m_pjPlatonic.translation_factor.setEnabled(false);
             };

             // Model setup
             trnModelCB.setEnabled(true);
             trfModelCB.setEnabled(true);
             pncModelCB.setEnabled(true);
             stlModelCB.setEnabled(true);
             m_pjPlatonic.m_parameter.setEnabled(true); 

             m_pjPlatonic.platonic.showVertexLabels(false);
             m_pjPlatonic.m_pwModel = new PwModel();
             m_pjPlatonic.m_pwModel.setGeometry(m_pjPlatonic.platonic);
             m_pjPlatonic.m_pwModel.setEnabledModeling(true);

             if (trnModelCB.getState() == true)
             {
                m_pjPlatonic.trn_fill = false;
                m_pjPlatonic.setModelingType(PwModel.TRUNCATE);
                m_pjPlatonic.platonic.copy(m_pjPlatonic.m_pwModel.getSavedGeometry()); 
                m_pjPlatonic.m_pwModel.truncate(m_pjPlatonic.platonic,m_pjPlatonic.m_parameter.getValue(), m_pjPlatonic.trn_fill);
             };
             if (trfModelCB.getState() == true)
             {
                m_pjPlatonic.trn_fill = true;
                m_pjPlatonic.setModelingType(PwModel.TRUNCATEFILL);
                m_pjPlatonic.platonic.copy(m_pjPlatonic.m_pwModel.getSavedGeometry()); 
                m_pjPlatonic.m_pwModel.truncate(m_pjPlatonic.platonic, m_pjPlatonic.m_parameter.getValue(), m_pjPlatonic.trn_fill);
             };
             if (pncModelCB.getState() == true)
             {
                m_pjPlatonic.setModelingType(PwModel.PUNCH);
                m_pjPlatonic.platonic.copy(m_pjPlatonic.m_pwModel.getSavedGeometry()); 
                m_pjPlatonic.m_pwModel.punch(m_pjPlatonic.platonic, m_pjPlatonic.m_parameter.getValue());
             };
             if (stlModelCB.getState() == true)
             {
                m_pjPlatonic.setModelingType(PwModel.STELLATE);
                m_pjPlatonic.platonic.copy(m_pjPlatonic.m_pwModel.getSavedGeometry()); 
                m_pjPlatonic.m_pwModel.stellate(m_pjPlatonic.platonic, m_pjPlatonic.m_parameter.getValue());
             };

             m_pjPlatonic.platonic.update(m_pjPlatonic.platonic);
          }
          else
          {
             trnModelCB.setEnabled(false);
             trfModelCB.setEnabled(false);
             pncModelCB.setEnabled(false);
             stlModelCB.setEnabled(false);
             m_pjPlatonic.m_parameter.setEnabled(false); 
             m_pjPlatonic.m_pwModel.cancel();
             m_pjPlatonic.platonic.showVertexLabels(true);
             m_pjPlatonic.restoreInitialConfiguration();
          };
       }; // End of if (source == enableModelCB)

       if (source == trnModelCB)
       {
          m_pjPlatonic.setModelingType(PwModel.TRUNCATE);
          updateModelSelectors();

          m_pjPlatonic.trn_fill = false;
          m_pjPlatonic.platonic.copy(m_pjPlatonic.m_pwModel.getSavedGeometry()); 
          m_pjPlatonic.m_pwModel.truncate(m_pjPlatonic.platonic, m_pjPlatonic.m_parameter.getValue(), m_pjPlatonic.trn_fill);
          m_pjPlatonic.platonic.update(m_pjPlatonic.platonic);
       }
       else
       {
          updateModelSelectors();
       }; // End of if (source == trnModelCB)

       if (source == trfModelCB)
       {
          m_pjPlatonic.setModelingType(PwModel.TRUNCATEFILL);
          updateModelSelectors();

          m_pjPlatonic.trn_fill = true;
          m_pjPlatonic.platonic.copy(m_pjPlatonic.m_pwModel.getSavedGeometry()); 
          m_pjPlatonic.m_pwModel.truncate(m_pjPlatonic.platonic, m_pjPlatonic.m_parameter.getValue(), m_pjPlatonic.trn_fill);
          m_pjPlatonic.platonic.update(m_pjPlatonic.platonic);
       }
       else
       {
          updateModelSelectors();
       }; // End of if (source == trfModelCB)

       if (source == pncModelCB)
       {
          m_pjPlatonic.setModelingType(PwModel.PUNCH);
          updateModelSelectors();

          m_pjPlatonic.platonic.copy(m_pjPlatonic.m_pwModel.getSavedGeometry()); 
          m_pjPlatonic.m_pwModel.punch(m_pjPlatonic.platonic, m_pjPlatonic.m_parameter.getValue());
          m_pjPlatonic.platonic.update(m_pjPlatonic.platonic);
       }
       else
       {
          updateModelSelectors();
       }; // End of if (source == pncModelCB)

       if (source == stlModelCB)
       {
          m_pjPlatonic.setModelingType(PwModel.STELLATE);
          updateModelSelectors();

          m_pjPlatonic.platonic.copy(m_pjPlatonic.m_pwModel.getSavedGeometry()); 
          m_pjPlatonic.m_pwModel.stellate(m_pjPlatonic.platonic, m_pjPlatonic.m_parameter.getValue());
          m_pjPlatonic.platonic.update(m_pjPlatonic.platonic);
       }
       else
       {
          updateModelSelectors();
       }; // End of if (source == stlModelCB)
    }; // End of itemStateChanged()


    //////////////////////////////////////////////////////////////////////////
    // Handle the proper state of the checkbox items in the model panel
    //////////////////////////////////////////////////////////////////////////
    protected void updateModelSelectors()
    {
       if (m_pjPlatonic.m_modelingType == PwModel.TRUNCATE)
       {
          trnModelCB.setState(true);
       }
       else
       {
          trnModelCB.setState(false);
       };

       if (m_pjPlatonic.m_modelingType == PwModel.TRUNCATEFILL)
       {
          trfModelCB.setState(true);
       }
       else
       {
          trfModelCB.setState(false);
       };

       if (m_pjPlatonic.m_modelingType == PwModel.PUNCH)
       {
          pncModelCB.setState(true);
       }
       else
       {
          pncModelCB.setState(false);
       };

       if (m_pjPlatonic.m_modelingType == PwModel.STELLATE)
       {
          stlModelCB.setState(true);
       }
       else
       {
          stlModelCB.setState(false);
       };
    }; // End of updateModelSelectors()
}; // End of PjPlatonic_IP class
